@@ -179,11 +179,12 @@ func isZero(v reflect.Value) bool {
 		if v.Type() == typeTime {
 			return v.Interface().(time.Time).IsZero()
 		}
-		info, err := getStructInfo(v.Type())
-		if err != nil {
-			panic("should never happen here: " + err.Error())
+		for i := v.NumField()-1; i >= 0; i-- {
+			if !isZero(v.Field(i)) {
+				return false
+			}
 		}
-		return info.Zero.Interface() == v.Interface()
+		return true
 	}
 	return false
 }
@@ -333,7 +334,7 @@ func (e *encoder) addElem(name string, v reflect.Value, minSize bool) {
 		if et.Kind() == reflect.Uint8 {
 			e.addElemName('\x05', name)
 			e.addBinary('\x00', v.Bytes())
-		} else if et == typeDocElem {
+		} else if et == typeDocElem || et == typeRawDocElem {
 			e.addElemName('\x03', name)
 			e.addDoc(v)
 		} else {
