@@ -63,6 +63,7 @@ type Session struct {
 	queryConfig  query
 	safeOp       *queryOp
 	syncTimeout  time.Duration
+	sockTimeout  time.Duration
 	defaultdb    string
 	dialAuth     *authInfo
 	auth         []authInfo
@@ -126,8 +127,8 @@ const defaultPrefetch = 0.25
 //
 // Dial will timeout after 10 seconds if a server isn't reached. The returned
 // session will timeout operations after one minute by default if servers
-// aren't available. To customize the timeout, see DialWithTimeout
-// and SetSyncTimeout.
+// aren't available. To customize the timeout, see DialWithTimeout,
+// SetSyncTimeout, and SetSocketTimeout.
 //
 // This method is generally called just once for a given cluster.  Further
 // sessions to the same cluster are then established using the New or Copy
@@ -172,9 +173,10 @@ const defaultPrefetch = 0.25
 //     http://www.mongodb.org/display/DOCS/Connections
 //
 func Dial(url string) (*Session, error) {
-	session, err := DialWithTimeout(url, 10*time.Second)
+	session, err := DialWithTimeout(url, 10 * time.Second)
 	if err == nil {
-		session.SetSyncTimeout(time.Minute)
+		session.SetSyncTimeout(1 * time.Minute)
+		session.SetSocketTimeout(1 * time.Minute)
 	}
 	return session, err
 }
@@ -334,9 +336,9 @@ func parseURL(url string) (*urlInfo, error) {
 	return info, nil
 }
 
-func newSession(consistency mode, cluster *mongoCluster, syncTimeout time.Duration) (session *Session) {
+func newSession(consistency mode, cluster *mongoCluster, timeout time.Duration) (session *Session) {
 	cluster.Acquire()
-	session = &Session{cluster_: cluster, syncTimeout: syncTimeout}
+	session = &Session{cluster_: cluster, syncTimeout: timeout, sockTimeout: timeout}
 	debugf("New session %p on cluster %p", session, cluster)
 	session.SetMode(consistency, true)
 	session.SetSafe(&Safe{})
@@ -976,7 +978,7 @@ func (s *Session) ResetIndexCache() {
 
 // New creates a new session with the same parameters as the original
 // session, including consistency, batch size, prefetching, safety mode,
-// etc. The returned session will use sockets from the poll, so there's
+// etc. The returned session will use sockets from the pool, so there's
 // a chance that writes just performed in another session may not yet
 // be visible.
 //
@@ -1119,6 +1121,20 @@ func (s *Session) SetSyncTimeout(d time.Duration) {
 	s.m.Unlock()
 }
 
+// SetSocketTimeout sets the amount of time to wait for a non-responding
+// socket to the database before it is forcefully closed.
+func (s *Session) SetSocketTimeout(d time.Duration) {
+	s.m.Lock()
+	s.sockTimeout = d
+	if s.masterSocket != nil {
+		s.masterSocket.SetTimeout(d)
+	}
+	if s.slaveSocket != nil {
+		s.slaveSocket.SetTimeout(d)
+	}
+	s.m.Unlock()
+}
+
 // SetBatch sets the default batch size used when fetching documents from the
 // database. It's possible to change this setting on a per-query basis as
 // well, using the Query.Batch method.
@@ -2586,8 +2602,11 @@ func (iter *Iter) acquireSocket() (*mongoSocket, error) {
 		// with Eventual sessions, if a Refresh is done, or if a
 		// monotonic session gets a write and shifts from secondary
 		// to primary. Our cursor is in a specific server, though.
+		iter.session.m.Lock()
+		sockTimeout := iter.session.sockTimeout
+		iter.session.m.Unlock()
 		socket.Release()
-		socket, _, err = iter.server.AcquireSocket(0)
+		socket, _, err = iter.server.AcquireSocket(0, sockTimeout)
 		if err != nil {
 			return nil, err
 		}
@@ -2822,7 +2841,7 @@ func (q *Query) MapReduce(job *MapReduce, result interface{}) (info *MapReduceIn
 		Map:        job.Map,
 		Reduce:     job.Reduce,
 		Finalize:   job.Finalize,
-		Out:        job.Out,
+		Out:        fixMROut(job.Out),
 		Scope:      job.Scope,
 		Verbose:    job.Verbose,
 		Query:      op.query,
@@ -2876,6 +2895,36 @@ func (q *Query) MapReduce(job *MapReduce, result interface{}) (info *MapReduceIn
 	return info, nil
 }
 
+// The "out" option in the MapReduce command must be ordered. This was
+// found after the implementation was accepting maps for a long time,
+// so rather than breaking the API, we'll fix the order if necessary.
+// Details about the order requirement may be seen in MongoDB's code:
+//
+//     http://goo.gl/L8jwJX
+//
+func fixMROut(out interface{}) interface{} {
+	outv := reflect.ValueOf(out)
+	if outv.Kind() != reflect.Map || outv.Type().Key() != reflect.TypeOf("") {
+		return out
+	}
+	outs := make(bson.D, outv.Len())
+
+	outTypeIndex := -1
+	for i, k := range outv.MapKeys() {
+		ks := k.String()
+		outs[i].Name = ks
+		outs[i].Value = outv.MapIndex(k).Interface()
+		switch ks {
+		case "normal", "replace", "merge", "reduce", "inline":
+			outTypeIndex = i
+		}
+	}
+	if outTypeIndex > 0 {
+		outs[0], outs[outTypeIndex] = outs[outTypeIndex], outs[0]
+	}
+	return outs
+}
+
 type Change struct {
 	Update    interface{} // The change document
 	Upsert    bool        // Whether to insert in case the document isn't found
@@ -3047,7 +3096,7 @@ func (s *Session) acquireSocket(slaveOk bool) (*mongoSocket, error) {
 	}
 
 	// Still not good.  We need a new socket.
-	sock, err := s.cluster().AcquireSocket(slaveOk && s.slaveOk, s.syncTimeout, s.queryConfig.op.serverTags)
+	sock, err := s.cluster().AcquireSocket(slaveOk && s.slaveOk, s.syncTimeout, s.sockTimeout, s.queryConfig.op.serverTags)
 	if err != nil {
 		return nil, err
 	}
