@@ -1,6 +1,6 @@
 
 var ports = [40001, 40002, 40011, 40012, 40013, 40021, 40022, 40023, 40041, 40101, 40102, 40103, 40201, 40202, 40203]
-var auth = [40002, 40103, 40203]
+var auth = [40002, 40103, 40203, 40031]
 
 for (var i in ports) {
     var port = ports[i]
