@@ -14,6 +14,10 @@ func SetAttemptStrategy(s *aws.AttemptStrategy) {
 	}
 }
 
+func AttemptStrategy() aws.AttemptStrategy {
+	return attempts
+}
+
 func Sign(auth aws.Auth, method, path string, params, headers map[string][]string) {
 	sign(auth, method, path, params, headers)
 }
