@@ -14,7 +14,7 @@ import (
 	"strings"
 )
 
-// Client represents a way ot communicating with a MAAS API instance.
+// Client represents a way to communicating with a MAAS API instance.
 // It is stateless, so it can have concurrent requests in progress.
 type Client struct {
 	APIURL *url.URL
@@ -48,6 +48,10 @@ func readAndClose(stream io.ReadCloser) ([]byte, error) {
 func (client Client) dispatchRequest(request *http.Request) ([]byte, error) {
 	client.Signer.OAuthSign(request)
 	httpClient := http.Client{}
+	// See https://code.google.com/p/go/issues/detail?id=4677
+	// We need to force the connection to close each time so that we don't
+	// hit the above Go bug.
+	request.Close = true
 	response, err := httpClient.Do(request)
 	if err != nil {
 		return nil, err
@@ -57,7 +61,7 @@ func (client Client) dispatchRequest(request *http.Request) ([]byte, error) {
 		return nil, err
 	}
 	if response.StatusCode < 200 || response.StatusCode > 299 {
-		msg := fmt.Errorf("gomaasapi: got error back from server: %v", response.Status)
+		msg := fmt.Errorf("gomaasapi: got error back from server: %v (%v)", response.Status, string(body))
 		return body, ServerError{error: msg, StatusCode: response.StatusCode}
 	}
 	return body, nil
