@@ -4,7 +4,9 @@ import (
 	"bytes"
 	"fmt"
 	"io/ioutil"
+
 	. "launchpad.net/gocheck"
+
 	"launchpad.net/goose/testing/httpsuite"
 	"net/http"
 	"testing"
@@ -156,6 +158,11 @@ func (s *HTTPClientTestSuite) TestJSONRequestSetsToken(c *C) {
 	c.Check(agent, Equals, "token")
 }
 
+func (s *HTTPClientTestSuite) TestHttpTransport(c *C) {
+	transport := http.DefaultTransport.(*http.Transport)
+	c.Assert(transport.DisableKeepAlives, Equals, true)
+}
+
 func (s *HTTPSClientTestSuite) TestDefaultClientRejectSelfSigned(c *C) {
 	_, _, client := s.setupLoopbackRequest()
 	req := &RequestData{ExpectedStatus: []int{http.StatusNoContent}}
