@@ -6,6 +6,8 @@ import (
 	"launchpad.net/goose/testservices/openstackservice"
 	"net/http"
 	"net/http/httptest"
+	"net/url"
+	"strings"
 )
 
 func registerLocalTests() {
@@ -39,7 +41,7 @@ func (s *localLiveSuite) SetUpSuite(c *C) {
 		Region:     "zone1.some region",
 		TenantName: "tenant",
 	}
-	openstack := openstackservice.New(s.cred)
+	openstack := openstackservice.New(s.cred, identity.AuthUserPass)
 	openstack.SetupHTTP(s.Mux)
 
 	s.LiveTests.SetUpSuite(c)
@@ -61,3 +63,22 @@ func (s *localLiveSuite) TearDownTest(c *C) {
 }
 
 // Additional tests to be run against the service double only go here.
+
+func (s *localLiveSuite) TestProductStreamsEndpoint(c *C) {
+	err := s.client.Authenticate()
+	c.Assert(err, IsNil)
+	serviceURL, err := s.client.MakeServiceURL("product-streams", nil)
+	c.Assert(err, IsNil)
+	_, err = url.Parse(serviceURL)
+	c.Assert(err, IsNil)
+	c.Assert(strings.HasSuffix(serviceURL, "/imagemetadata"), Equals, true)
+}
+
+func (s *localLiveSuite) TestJujuToolsEndpoint(c *C) {
+	err := s.client.Authenticate()
+	c.Assert(err, IsNil)
+	serviceURL, err := s.client.MakeServiceURL("juju-tools", nil)
+	c.Assert(err, IsNil)
+	_, err = url.Parse(serviceURL)
+	c.Assert(err, IsNil)
+}
