@@ -184,7 +184,11 @@ func (c *Client) ListServers(filter *Filter) ([]Entity, error) {
 	var resp struct {
 		Servers []Entity
 	}
-	requestData := goosehttp.RequestData{RespValue: &resp, Params: &filter.v, ExpectedStatus: []int{http.StatusOK}}
+	var params *url.Values
+	if filter != nil {
+		params = &filter.v
+	}
+	requestData := goosehttp.RequestData{RespValue: &resp, Params: params, ExpectedStatus: []int{http.StatusOK}}
 	err := c.client.SendRequest(client.GET, "compute", apiServers, &requestData)
 	if err != nil {
 		return nil, errors.Newf(err, "failed to get list of servers")
@@ -248,7 +252,11 @@ func (c *Client) ListServersDetail(filter *Filter) ([]ServerDetail, error) {
 	var resp struct {
 		Servers []ServerDetail
 	}
-	requestData := goosehttp.RequestData{RespValue: &resp, Params: &filter.v}
+	var params *url.Values
+	if filter != nil {
+		params = &filter.v
+	}
+	requestData := goosehttp.RequestData{RespValue: &resp, Params: params}
 	err := c.client.SendRequest(client.GET, "compute", apiServersDetail, &requestData)
 	if err != nil {
 		return nil, errors.Newf(err, "failed to get list of server details")
@@ -288,6 +296,15 @@ type SecurityGroupName struct {
 	Name string `json:"name"`
 }
 
+// ServerNetworks sets what networks a server should be connected to on boot.
+// - FixedIp may be supplied only when NetworkId is also given.
+// - PortId may be supplied only if neither NetworkId or FixedIp is set.
+type ServerNetworks struct {
+	NetworkId string `json:"uuid,omitempty"`
+	FixedIp   string `json:"fixed_ip,omitempty"`
+	PortId    string `json:"port,omitempty"`
+}
+
 // RunServerOpts defines required and optional arguments for RunServer().
 type RunServerOpts struct {
 	Name               string              `json:"name"`            // Required
@@ -295,6 +312,7 @@ type RunServerOpts struct {
 	ImageId            string              `json:"imageRef"`        // Required
 	UserData           []byte              `json:"user_data"`       // Optional
 	SecurityGroupNames []SecurityGroupName `json:"security_groups"` // Optional
+	Networks           []ServerNetworks    `json:"networks"`        // Optional
 }
 
 // RunServer creates a new server, based on the given RunServerOpts.
