@@ -297,7 +297,7 @@ func (h *novaHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 	var resp http.Handler
-	if _, ok := err.(*testservices.RateLimitExceededError); ok {
+	if err == testservices.RateLimitExceededError {
 		resp = errRateLimitExceeded
 	} else if err == testservices.NoMoreFloatingIPs {
 		resp = errNoMoreFloatingIPs
@@ -539,6 +539,7 @@ func (n *Nova) handleRunServer(body []byte, w http.ResponseWriter, r *http.Reque
 			Name           string
 			Metadata       map[string]string
 			SecurityGroups []map[string]string `json:"security_groups"`
+			Networks       []map[string]string
 		}
 	}
 	if err := json.Unmarshal(body, &req); err != nil {
@@ -563,10 +564,6 @@ func (n *Nova) handleRunServer(body []byte, w http.ResponseWriter, r *http.Reque
 	if len(req.Server.SecurityGroups) > 0 {
 		for _, group := range req.Server.SecurityGroups {
 			groupName := group["name"]
-			if groupName == "default" {
-				// assume default security group exists
-				continue
-			}
 			if sg, err := n.securityGroupByName(groupName); err != nil {
 				return noGroupError(groupName, n.TenantId)
 			} else {
@@ -574,6 +571,14 @@ func (n *Nova) handleRunServer(body []byte, w http.ResponseWriter, r *http.Reque
 			}
 		}
 	}
+	// TODO(gz) some kind of sane handling of networks
+	for _, network := range req.Server.Networks {
+		networkId := network["uuid"]
+		_, ok := n.networks[networkId]
+		if !ok {
+			return errNotFoundJSON
+		}
+	}
 	// TODO(dimitern) - 2013-02-11 bug=1121684
 	// make sure flavor/image exist (if needed)
 	flavor := nova.FlavorDetail{Id: req.Server.FlavorRef}
@@ -991,6 +996,27 @@ func (n *Nova) handleFloatingIPs(w http.ResponseWriter, r *http.Request) error {
 	return fmt.Errorf("unknown request method %q for %s", r.Method, r.URL.Path)
 }
 
+// handleNetworks handles the os-networks HTTP API.
+func (n *Nova) handleNetworks(w http.ResponseWriter, r *http.Request) error {
+	switch r.Method {
+	case "GET":
+		if ipId := path.Base(r.URL.Path); ipId != "os-networks" {
+			// TODO(gz): handle listing a single group
+			return errNotFoundJSON
+		}
+		nets := n.allNetworks()
+		if len(nets) == 0 {
+			nets = []nova.Network{}
+		}
+		resp := struct {
+			Network []nova.Network `json:"networks"`
+		}{nets}
+		return sendJSON(http.StatusOK, resp, w, r)
+		// TODO(gz): proper handling of other methods
+	}
+	return fmt.Errorf("unknown request method %q for %s", r.Method, r.URL.Path)
+}
+
 // setupHTTP attaches all the needed handlers to provide the HTTP API.
 func (n *Nova) SetupHTTP(mux *http.ServeMux) {
 	handlers := map[string]http.Handler{
@@ -1004,6 +1030,7 @@ func (n *Nova) SetupHTTP(mux *http.ServeMux) {
 		"/$v/$t/os-security-groups":      n.handler((*Nova).handleSecurityGroups),
 		"/$v/$t/os-security-group-rules": n.handler((*Nova).handleSecurityGroupRules),
 		"/$v/$t/os-floating-ips":         n.handler((*Nova).handleFloatingIPs),
+		"/$v/$t/os-networks":             n.handler((*Nova).handleNetworks),
 	}
 	for path, h := range handlers {
 		path = strings.Replace(path, "$v", n.VersionPath, 1)
