@@ -223,6 +223,15 @@ var unmarshalTests = []struct {
 	}, {
 		"a: 1",
 		&struct{ A int }{1},
+	}, {
+		"a: 1",
+		&struct{ A float64 }{1},
+	}, {
+		"a: 1.0",
+		&struct{ A int }{1},
+	}, {
+		"a: 1.0",
+		&struct{ A uint }{1},
 	}, {
 		"a: [1, 2]",
 		&struct{ A []int }{[]int{1, 2}},
@@ -393,6 +402,7 @@ var unmarshalErrorTests = []struct {
 	{"v:\n- [A,", "YAML error: line 2: did not find expected node content"},
 	{"a: *b\n", "YAML error: Unknown anchor 'b' referenced"},
 	{"a: &a\n  b: *a\n", "YAML error: Anchor 'a' value contains itself"},
+	{"value: -", "YAML error: block sequence entries are not allowed in this context"},
 }
 
 func (s *S) TestUnmarshalErrors(c *C) {
