@@ -1,23 +1,26 @@
-// Copyright 2013 Canonical Ltd.  This software is licensed under the
+// Copyright 2013-2014 Canonical Ltd.  This software is licensed under the
 // GNU Lesser General Public License version 3 (see the file COPYING).
 
-// Define the various role sizes and their attributes.
+// Define the role sizes available in Azure.
 
 package gwacl
 
-// RoleSize is a representation of the data available in the Azure
+// RoleSize is a representation of the machine specs available in the Azure
 // documentation here:
 // http://msdn.microsoft.com/en-us/library/windowsazure/dn197896.aspx and
 //
 // Pricing from here:
-// http://www.windowsazure.com/en-us/pricing/details/virtual-machines/
+// http://www.windowsazure.com/en-us/pricing/details/cloud-services/
+//
+// Detailed specifications here:
+// http://msdn.microsoft.com/en-us/library/windowsazure/dn197896.aspx
 //
 // Our specifications may be inaccurate or out of date.  When in doubt, check!
 //
 // The Disk Space values are only the maxumim permitted; actual space is
 // determined by the OS image being used.
 //
-// Sizes and costs last updated 2013-09-24.
+// Sizes and costs last updated 2014-01-31.
 type RoleSize struct {
     Name             string
     CpuCores         uint64
@@ -52,7 +55,7 @@ var RoleSizes = []RoleSize{
         OSDiskSpaceCloud: 224 * GB,
         OSDiskSpaceVirt:  70 * GB,
         MaxDataDisks:     2,
-        Cost:             9,
+        Cost:             8,
     }, {
         Name:             "Medium",
         CpuCores:         2,
@@ -60,7 +63,7 @@ var RoleSizes = []RoleSize{
         OSDiskSpaceCloud: 489 * GB,
         OSDiskSpaceVirt:  135 * GB,
         MaxDataDisks:     4,
-        Cost:             18,
+        Cost:             16,
     }, {
         Name:             "Large",
         CpuCores:         4,
@@ -68,23 +71,23 @@ var RoleSizes = []RoleSize{
         OSDiskSpaceCloud: 999 * GB,
         OSDiskSpaceVirt:  285 * GB,
         MaxDataDisks:     8,
-        Cost:             36,
+        Cost:             32,
     }, {
         Name:             "ExtraLarge",
         CpuCores:         8,
         Mem:              14 * GB,
         OSDiskSpaceCloud: 2039 * GB,
-        OSDiskSpaceVirt:  65 * GB,
+        OSDiskSpaceVirt:  605 * GB,
         MaxDataDisks:     16,
-        Cost:             72,
+        Cost:             64,
     }, {
         Name:             "A5",
         CpuCores:         2,
         Mem:              14 * GB,
-        OSDiskSpaceCloud: 489 * GB, // Estimate; not yet announced.
-        OSDiskSpaceVirt:  285 * GB, // Estimate; not yet announced.
-        MaxDataDisks:     2,        // Estimate; not yet announced.
-        Cost:             51,
+        OSDiskSpaceCloud: 489 * GB,
+        OSDiskSpaceVirt:  135 * GB,
+        MaxDataDisks:     4,
+        Cost:             35,
     }, {
         Name:             "A6",
         CpuCores:         4,
@@ -92,15 +95,31 @@ var RoleSizes = []RoleSize{
         OSDiskSpaceCloud: 999 * GB,
         OSDiskSpaceVirt:  285 * GB,
         MaxDataDisks:     8,
-        Cost:             102,
+        Cost:             71,
     }, {
         Name:             "A7",
         CpuCores:         8,
         Mem:              56 * GB,
         OSDiskSpaceCloud: 2039 * GB,
-        OSDiskSpaceVirt:  65 * GB,
+        OSDiskSpaceVirt:  605 * GB,
         MaxDataDisks:     16,
-        Cost:             204,
+        Cost:             141,
+    }, {
+        Name:             "A8",
+        CpuCores:         8,
+        Mem:              56 * GB,
+        OSDiskSpaceCloud: 2039 * GB, // Estimate; not yet announced.
+        OSDiskSpaceVirt:  605 * GB,  // Estimate; not yet announced.
+        MaxDataDisks:     16,        // Estimate; not yet announced.
+        Cost:             245,
+    }, {
+        Name:             "A9",
+        CpuCores:         16,
+        Mem:              112 * GB,
+        OSDiskSpaceCloud: 2039 * GB, // Estimate; not yet announced.
+        OSDiskSpaceVirt:  605 * GB,  // Estimate; not yet announced.
+        MaxDataDisks:     16,        // Estimate; not yet announced.
+        Cost:             490,
     },
 }
 
