@@ -41,6 +41,17 @@ var headersToSign = []string{
     "Range",
 }
 
+func init() {
+    // See https://code.google.com/p/go/issues/detail?id=4677
+    // We need to force the connection to close each time so that we don't
+    // hit the above Go bug.
+    roundTripper := http.DefaultClient.Transport
+    if transport, ok := roundTripper.(*http.Transport); ok {
+        transport.DisableKeepAlives = true
+    }
+    http.DefaultTransport.(*http.Transport).DisableKeepAlives = true
+}
+
 // sign returns the base64-encoded HMAC-SHA256 signature of the given string
 // using the given base64-encoded key.
 func sign(accountKey, signable string) (string, error) {
