@@ -4,6 +4,7 @@
 package gwacl
 
 import (
+    "errors"
     "fmt"
     "launchpad.net/gwacl/fork/http"
     "launchpad.net/gwacl/fork/tls"
@@ -19,6 +20,11 @@ type x509Session struct {
     retryPolicy    RetryPolicy
 }
 
+// httpRedirectErr is a unique error used to prevent
+// net/http from automatically following redirects.
+// See commentary on CheckRedirect in newX509Session.
+var httpRedirectErr = errors.New("redirect")
+
 // newX509Session creates and returns a new x509Session based on credentials
 // and X509 certificate files.
 // For testing purposes, certFile can be passed as the empty string and it
@@ -43,6 +49,13 @@ func newX509Session(subscriptionId, certFile, location string, retryPolicy Retry
             // hit the above Go bug.
             DisableKeepAlives: true,
         },
+        // See https://code.google.com/p/go/issues/detail?id=4800
+        // We need to follow temporary redirects (307s), while
+        // retaining headers. We also need to follow redirects
+        // for POST and DELETE automatically.
+        CheckRedirect: func(req *http.Request, via []*http.Request) error {
+            return httpRedirectErr
+        },
     }
 
     endpointURL := GetEndpoint(location).ManagementAPI()
