@@ -1,11 +1,16 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package cmd_test
 
 import (
+	"os"
+
 	"launchpad.net/gnuflag"
-	. "launchpad.net/gocheck"
+	gc "launchpad.net/gocheck"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/testing"
-	"os"
 )
 
 type FileVarSuite struct {
@@ -14,37 +19,37 @@ type FileVarSuite struct {
 	InvalidPath string // invalid path refers to a file which is not readable
 }
 
-var _ = Suite(&FileVarSuite{})
+var _ = gc.Suite(&FileVarSuite{})
 
-func (s *FileVarSuite) SetUpTest(c *C) {
+func (s *FileVarSuite) SetUpTest(c *gc.C) {
 	s.ctx = testing.Context(c)
 	s.ValidPath = s.ctx.AbsPath("valid.yaml")
 	s.InvalidPath = s.ctx.AbsPath("invalid.yaml")
 	f, err := os.Create(s.ValidPath)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	f.Close()
 	f, err = os.Create(s.InvalidPath)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	f.Close()
 	err = os.Chmod(s.InvalidPath, 0) // make unreadable
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 }
 
-func (s *FileVarSuite) TestValidFileVar(c *C) {
+func (s *FileVarSuite) TestValidFileVar(c *gc.C) {
 	fs, config := fs()
 	err := fs.Parse(false, []string{"--config", s.ValidPath})
-	c.Assert(err, IsNil)
-	c.Assert(config.Path, Equals, s.ValidPath)
+	c.Assert(err, gc.IsNil)
+	c.Assert(config.Path, gc.Equals, s.ValidPath)
 	_, err = config.Read(s.ctx)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 }
 
-func (s *FileVarSuite) TestInvalidFileVar(c *C) {
+func (s *FileVarSuite) TestInvalidFileVar(c *gc.C) {
 	fs, config := fs()
 	err := fs.Parse(false, []string{"--config", s.InvalidPath})
-	c.Assert(config.Path, Equals, s.InvalidPath)
+	c.Assert(config.Path, gc.Equals, s.InvalidPath)
 	_, err = config.Read(s.ctx)
-	c.Assert(err, ErrorMatches, "*permission denied")
+	c.Assert(err, gc.ErrorMatches, "*permission denied")
 }
 
 func fs() (*gnuflag.FlagSet, *cmd.FileVar) {
