@@ -4,8 +4,10 @@
 package main
 
 import (
+	jc "github.com/juju/testing/checkers"
 	gc "launchpad.net/gocheck"
 
+	"launchpad.net/juju-core/errors"
 	jujutesting "launchpad.net/juju-core/juju/testing"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/testing"
@@ -22,7 +24,7 @@ func runDestroyMachine(c *gc.C, args ...string) error {
 	return err
 }
 
-func (s *DestroyMachineSuite) TestDestroyMachine(c *gc.C) {
+func (s *DestroyMachineSuite) TestDestroyMachineWithUnit(c *gc.C) {
 	// Create a machine running a unit.
 	testing.Charms.BundlePath(s.SeriesPath, "riak")
 	err := runDeploy(c, "local:riak", "riak")
@@ -38,19 +40,16 @@ func (s *DestroyMachineSuite) TestDestroyMachine(c *gc.C) {
 	// Try to destroy the machine and fail.
 	err = runDestroyMachine(c, "0")
 	c.Assert(err, gc.ErrorMatches, `no machines were destroyed: machine 0 has unit "riak/0" assigned`)
+}
 
-	// Remove the unit, and try to destroy the machine along with another that
-	// doesn't exist; check that the machine is destroyed, but the missing one
-	// is warned about.
-	err = u.Destroy()
-	c.Assert(err, gc.IsNil)
-	err = u.EnsureDead()
-	c.Assert(err, gc.IsNil)
-	err = u.Remove()
+func (s *DestroyMachineSuite) TestDestroyEmptyMachine(c *gc.C) {
+	// Destroy an empty machine alongside a state server; only the empty machine
+	// gets destroyed.
+	m0, err := s.State.AddMachine("quantal", state.JobHostUnits)
 	c.Assert(err, gc.IsNil)
 	err = runDestroyMachine(c, "0", "1")
 	c.Assert(err, gc.ErrorMatches, `some machines were not destroyed: machine 1 does not exist`)
-	m0, err := s.State.Machine("0")
+	err = m0.Refresh()
 	c.Assert(err, gc.IsNil)
 	c.Assert(m0.Life(), gc.Equals, state.Dying)
 
@@ -60,25 +59,73 @@ func (s *DestroyMachineSuite) TestDestroyMachine(c *gc.C) {
 	err = m0.Refresh()
 	c.Assert(err, gc.IsNil)
 	c.Assert(m0.Life(), gc.Equals, state.Dying)
+}
 
-	// As is destroying a Dead machine; destroying it alongside a JobManageEnviron
-	// machine complains only about the JMA machine.
-	err = m0.EnsureDead()
+func (s *DestroyMachineSuite) TestDestroyDeadMachine(c *gc.C) {
+	// Destroying a Dead machine is a no-op; destroying it alongside a JobManageEnviron
+	m0, err := s.State.AddMachine("quantal", state.JobManageEnviron)
 	c.Assert(err, gc.IsNil)
-	m1, err := s.State.AddMachine("quantal", state.JobManageEnviron)
+	// machine complains only about the JME machine.
+	m1, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, gc.IsNil)
+	err = m1.EnsureDead()
 	c.Assert(err, gc.IsNil)
 	err = runDestroyMachine(c, "0", "1")
-	c.Assert(err, gc.ErrorMatches, `some machines were not destroyed: machine 1 is required by the environment`)
-	err = m0.Refresh()
-	c.Assert(err, gc.IsNil)
-	c.Assert(m0.Life(), gc.Equals, state.Dead)
+	c.Assert(err, gc.ErrorMatches, `some machines were not destroyed: machine 0 is required by the environment`)
 	err = m1.Refresh()
 	c.Assert(err, gc.IsNil)
-	c.Assert(m1.Life(), gc.Equals, state.Alive)
+	c.Assert(m1.Life(), gc.Equals, state.Dead)
+	err = m1.Refresh()
+	c.Assert(err, gc.IsNil)
+	c.Assert(m0.Life(), gc.Equals, state.Alive)
+}
 
+func (s *DestroyMachineSuite) TestForce(c *gc.C) {
+	// Create a manager machine.
+	m0, err := s.State.AddMachine("quantal", state.JobManageEnviron)
+	c.Assert(err, gc.IsNil)
+
+	// Create a machine running a unit.
+	testing.Charms.BundlePath(s.SeriesPath, "riak")
+	err = runDeploy(c, "local:riak", "riak")
+	c.Assert(err, gc.IsNil)
+
+	// Get the state entities to allow sane testing.
+	u, err := s.State.Unit("riak/0")
+	c.Assert(err, gc.IsNil)
+	m1, err := s.State.Machine("1")
+	c.Assert(err, gc.IsNil)
+
+	// Try to force-destroy the machines.
+	err = runDestroyMachine(c, "0", "1", "--force")
+	c.Assert(err, gc.ErrorMatches, `some machines were not destroyed: machine 0 is required by the environment`)
+
+	// Clean up, check state.
+	err = s.State.Cleanup()
+	c.Assert(err, gc.IsNil)
+	err = u.Refresh()
+	c.Assert(err, jc.Satisfies, errors.IsNotFoundError)
+
+	err = m1.Refresh()
+	c.Assert(err, gc.IsNil)
+	c.Assert(m1.Life(), gc.Equals, state.Dead)
+
+	err = m0.Refresh()
+	c.Assert(err, gc.IsNil)
+	c.Assert(m0.Life(), gc.Equals, state.Alive)
+}
+
+func (s *DestroyMachineSuite) TestBadArgs(c *gc.C) {
 	// Check invalid args.
-	err = runDestroyMachine(c)
+	err := runDestroyMachine(c)
 	c.Assert(err, gc.ErrorMatches, `no machines specified`)
 	err = runDestroyMachine(c, "1", "2", "nonsense", "rubbish")
 	c.Assert(err, gc.ErrorMatches, `invalid machine id "nonsense"`)
 }
+
+func (s *DestroyMachineSuite) TestEnvironmentArg(c *gc.C) {
+	_, err := s.State.AddMachine("quantal", state.JobHostUnits)
+	c.Assert(err, gc.IsNil)
+	err = runDestroyMachine(c, "0", "-e", "dummyenv")
+	c.Assert(err, gc.IsNil)
+}
