@@ -11,6 +11,7 @@ import (
 
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
+	"launchpad.net/juju-core/state/api/params"
 )
 
 // GetEnvironmentCommand is able to output either the entire environment or
@@ -53,31 +54,51 @@ func (c *GetEnvironmentCommand) Init(args []string) (err error) {
 	return
 }
 
-func (c *GetEnvironmentCommand) Run(ctx *cmd.Context) error {
+// environmentGet1dot16 runs matches client.EnvironmentGet using a direct DB
+// connection to maintain compatibility with an API server running 1.16 or
+// older (when EnvironmentGet was not available). This fallback can be removed
+// when we no longer maintain 1.16 compatibility.
+func (c *GetEnvironmentCommand) environmentGet1dot16() (map[string]interface{}, error) {
 	conn, err := juju.NewConnFromName(c.EnvName)
 	if err != nil {
-		return err
+		return nil, err
 	}
 	defer conn.Close()
 
 	// Get the existing environment config from the state.
 	config, err := conn.State.EnvironConfig()
 	if err != nil {
-		return err
+		return nil, err
 	}
 	attrs := config.AllAttrs()
+	return attrs, nil
+}
 
-	// If no key specified, write out the whole lot.
-	if c.key == "" {
-		return c.out.Write(ctx, attrs)
+func (c *GetEnvironmentCommand) Run(ctx *cmd.Context) error {
+	client, err := juju.NewAPIClientFromName(c.EnvName)
+	if err != nil {
+		return err
+	}
+	defer client.Close()
+
+	attrs, err := client.EnvironmentGet()
+	if params.IsCodeNotImplemented(err) {
+		logger.Infof("EnvironmentGet not supported by the API server, " +
+			"falling back to 1.16 compatibility mode (direct DB access)")
+		attrs, err = c.environmentGet1dot16()
+	}
+	if err != nil {
+		return err
 	}
 
-	value, found := attrs[c.key]
-	if found {
-		return c.out.Write(ctx, value)
+	if c.key != "" {
+		if value, found := attrs[c.key]; found {
+			return c.out.Write(ctx, value)
+		}
+		return fmt.Errorf("Key %q not found in %q environment.", c.key, attrs["name"])
 	}
-
-	return fmt.Errorf("Key %q not found in %q environment.", c.key, config.Name())
+	// If key is empty, write out the whole lot.
+	return c.out.Write(ctx, attrs)
 }
 
 type attributes map[string]interface{}
@@ -129,31 +150,96 @@ func (c *SetEnvironmentCommand) Init(args []string) (err error) {
 	return nil
 }
 
-func (c *SetEnvironmentCommand) Run(ctx *cmd.Context) error {
+// run1dot16 runs matches client.EnvironmentSet using a direct DB
+// connection to maintain compatibility with an API server running 1.16 or
+// older (when EnvironmentSet was not available). This fallback can be removed
+// when we no longer maintain 1.16 compatibility.
+// This content was copied from SetEnvironmentCommand.Run in 1.16
+func (c *SetEnvironmentCommand) run1dot16() error {
 	conn, err := juju.NewConnFromName(c.EnvName)
 	if err != nil {
 		return err
 	}
 	defer conn.Close()
 
-	// Here is the magic around setting the attributes:
-	// TODO(thumper): get this magic under test somewhere, and update other call-sites to use it.
-	// Get the existing environment config from the state.
-	oldConfig, err := conn.State.EnvironConfig()
+	// Update state config with new values
+	return conn.State.UpdateEnvironConfig(c.values, nil, nil)
+}
+
+func (c *SetEnvironmentCommand) Run(ctx *cmd.Context) error {
+	client, err := juju.NewAPIClientFromName(c.EnvName)
 	if err != nil {
 		return err
 	}
-	// Apply the attributes specified for the command to the state config.
-	newConfig, err := oldConfig.Apply(c.values)
+	defer client.Close()
+
+	err = client.EnvironmentSet(c.values)
+	if params.IsCodeNotImplemented(err) {
+		logger.Infof("EnvironmentSet not supported by the API server, " +
+			"falling back to 1.16 compatibility mode (direct DB access)")
+		err = c.run1dot16()
+	}
+	return err
+}
+
+// UnsetEnvironment
+type UnsetEnvironmentCommand struct {
+	cmd.EnvCommandBase
+	keys []string
+}
+
+const unsetEnvHelpDoc = `
+Reset one or more the environment configuration attributes to its default
+value in a running Juju instance.  Attributes without defaults are removed,
+and attempting to remove a required attribute with no default will result
+in an error.
+
+Multiple attributes may be removed at once; keys are space-separated.
+`
+
+func (c *UnsetEnvironmentCommand) Info() *cmd.Info {
+	return &cmd.Info{
+		Name:    "unset-environment",
+		Args:    "<environment key> ...",
+		Purpose: "unset environment values",
+		Doc:     strings.TrimSpace(unsetEnvHelpDoc),
+		Aliases: []string{"unset-env"},
+	}
+}
+
+func (c *UnsetEnvironmentCommand) Init(args []string) (err error) {
+	if len(args) == 0 {
+		return fmt.Errorf("No keys specified")
+	}
+	c.keys = args
+	return nil
+}
+
+// run1dot16 runs matches client.EnvironmentUnset using a direct DB
+// connection to maintain compatibility with an API server running 1.16 or
+// older (when EnvironmentUnset was not available). This fallback can be removed
+// when we no longer maintain 1.16 compatibility.
+func (c *UnsetEnvironmentCommand) run1dot16() error {
+	conn, err := juju.NewConnFromName(c.EnvName)
 	if err != nil {
 		return err
 	}
-	// Now validate this new config against the existing config via the provider.
-	provider := conn.Environ.Provider()
-	newProviderConfig, err := provider.Validate(newConfig, oldConfig)
+	defer conn.Close()
+	return conn.State.UpdateEnvironConfig(nil, c.keys, nil)
+}
+
+func (c *UnsetEnvironmentCommand) Run(ctx *cmd.Context) error {
+	client, err := juju.NewAPIClientFromName(c.EnvName)
 	if err != nil {
 		return err
+	}
+	defer client.Close()
+
+	err = client.EnvironmentUnset(c.keys...)
+	if params.IsCodeNotImplemented(err) {
+		logger.Infof("EnvironmentUnset not supported by the API server, " +
+			"falling back to 1.16 compatibility mode (direct DB access)")
+		err = c.run1dot16()
 	}
-	// Now try to apply the new validated config.
-	return conn.State.SetEnvironConfig(newProviderConfig)
+	return err
 }
