@@ -7,10 +7,13 @@ import (
 	"fmt"
 	"strings"
 
+	jc "github.com/juju/testing/checkers"
 	gc "launchpad.net/gocheck"
 
+	"launchpad.net/juju-core/environs/config"
 	jujutesting "launchpad.net/juju-core/juju/testing"
 	"launchpad.net/juju-core/provider/dummy"
+	_ "launchpad.net/juju-core/provider/local"
 	"launchpad.net/juju-core/testing"
 )
 
@@ -121,12 +124,31 @@ func (s *SetEnvironmentSuite) TestInit(c *gc.C) {
 }
 
 func (s *SetEnvironmentSuite) TestChangeDefaultSeries(c *gc.C) {
-	_, err := testing.RunCommand(c, &SetEnvironmentCommand{}, []string{"default-series=raring"})
+	// default-series not set
+	stateConfig, err := s.State.EnvironConfig()
+	c.Assert(err, gc.IsNil)
+	series, ok := stateConfig.DefaultSeries()
+	c.Assert(ok, gc.Equals, true)
+	c.Assert(series, gc.Equals, "precise") // default-series set in RepoSuite.SetUpTest
+
+	_, err = testing.RunCommand(c, &SetEnvironmentCommand{}, []string{"default-series=raring"})
+	c.Assert(err, gc.IsNil)
+
+	stateConfig, err = s.State.EnvironConfig()
+	c.Assert(err, gc.IsNil)
+	series, ok = stateConfig.DefaultSeries()
+	c.Assert(ok, gc.Equals, true)
+	c.Assert(series, gc.Equals, "raring")
+	c.Assert(config.PreferredSeries(stateConfig), gc.Equals, "raring")
+}
+
+func (s *SetEnvironmentSuite) TestChangeBooleanAttribute(c *gc.C) {
+	_, err := testing.RunCommand(c, &SetEnvironmentCommand{}, []string{"ssl-hostname-verification=false"})
 	c.Assert(err, gc.IsNil)
 
 	stateConfig, err := s.State.EnvironConfig()
 	c.Assert(err, gc.IsNil)
-	c.Assert(stateConfig.DefaultSeries(), gc.Equals, "raring")
+	c.Assert(stateConfig.SSLHostnameVerification(), gc.Equals, false)
 }
 
 func (s *SetEnvironmentSuite) TestChangeMultipleValues(c *gc.C) {
@@ -154,15 +176,87 @@ func (s *SetEnvironmentSuite) TestChangeAsCommandPair(c *gc.C) {
 
 var immutableConfigTests = map[string]string{
 	"name":          "foo",
-	"type":          "foo",
+	"type":          "local",
 	"firewall-mode": "global",
+	"state-port":    "1",
+	"api-port":      "666",
 }
 
 func (s *SetEnvironmentSuite) TestImmutableConfigValues(c *gc.C) {
 	for name, value := range immutableConfigTests {
 		param := fmt.Sprintf("%s=%s", name, value)
 		_, err := testing.RunCommand(c, &SetEnvironmentCommand{}, []string{param})
-		errorPattern := fmt.Sprintf("cannot change %s from .* to %q", name, value)
+		errorPattern := fmt.Sprintf("cannot change %s from .* to [\"]?%v[\"]?", name, value)
 		c.Assert(err, gc.ErrorMatches, errorPattern)
 	}
 }
+
+type UnsetEnvironmentSuite struct {
+	jujutesting.RepoSuite
+}
+
+var _ = gc.Suite(&UnsetEnvironmentSuite{})
+
+var unsetEnvTests = []struct {
+	args       []string
+	err        string
+	expected   attributes
+	unexpected []string
+}{
+	{
+		args: []string{},
+		err:  "No keys specified",
+	}, {
+		args:       []string{"xyz", "xyz"},
+		unexpected: []string{"xyz"},
+	}, {
+		args: []string{"type", "xyz"},
+		err:  "type: expected string, got nothing",
+		expected: attributes{
+			"type": "dummy",
+			"xyz":  123,
+		},
+	}, {
+		args: []string{"syslog-port"},
+		expected: attributes{
+			"syslog-port": config.DefaultSyslogPort,
+		},
+	}, {
+		args:       []string{"xyz2", "xyz"},
+		unexpected: []string{"xyz"},
+	},
+}
+
+func (s *UnsetEnvironmentSuite) initConfig(c *gc.C) {
+	err := s.State.UpdateEnvironConfig(map[string]interface{}{
+		"syslog-port": 1234,
+		"xyz":         123,
+	}, nil, nil)
+	c.Assert(err, gc.IsNil)
+}
+
+func (s *UnsetEnvironmentSuite) TestUnsetEnvironment(c *gc.C) {
+	for _, t := range unsetEnvTests {
+		c.Logf("testing unset-env %v", t.args)
+		s.initConfig(c)
+		_, err := testing.RunCommand(c, &UnsetEnvironmentCommand{}, t.args)
+		if t.err != "" {
+			c.Assert(err, gc.ErrorMatches, t.err)
+		} else {
+			c.Assert(err, gc.IsNil)
+		}
+		if len(t.expected)+len(t.unexpected) != 0 {
+			stateConfig, err := s.State.EnvironConfig()
+			c.Assert(err, gc.IsNil)
+			for k, v := range t.expected {
+				vstate, ok := stateConfig.AllAttrs()[k]
+				c.Assert(ok, jc.IsTrue)
+				c.Assert(vstate, gc.Equals, v)
+			}
+			for _, k := range t.unexpected {
+				_, ok := stateConfig.AllAttrs()[k]
+				c.Assert(ok, jc.IsFalse)
+			}
+		}
+	}
+}
