@@ -16,14 +16,11 @@ import (
 
 	"launchpad.net/gnuflag"
 	gc "launchpad.net/gocheck"
-	"launchpad.net/loggo"
 
 	"launchpad.net/juju-core/cmd"
-	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/juju/osenv"
 	_ "launchpad.net/juju-core/provider/dummy"
 	"launchpad.net/juju-core/testing"
-	"launchpad.net/juju-core/testing/testbase"
 	"launchpad.net/juju-core/version"
 )
 
@@ -32,7 +29,7 @@ func TestPackage(t *stdtesting.T) {
 }
 
 type MainSuite struct {
-	testbase.LoggingSuite
+	testing.FakeHomeSuite
 }
 
 var _ = gc.Suite(&MainSuite{})
@@ -52,7 +49,7 @@ func TestRunMain(t *stdtesting.T) {
 func badrun(c *gc.C, exit int, args ...string) string {
 	localArgs := append([]string{"-test.run", "TestRunMain", "-run-main", "--", "juju"}, args...)
 	ps := exec.Command(os.Args[0], localArgs...)
-	ps.Env = append(os.Environ(), osenv.JujuHome+"="+config.JujuHome())
+	ps.Env = append(os.Environ(), osenv.JujuHomeEnvKey+"="+osenv.JujuHome())
 	output, err := ps.CombinedOutput()
 	c.Logf("command output: %q", output)
 	if exit != 0 {
@@ -79,17 +76,13 @@ func syncToolsHelpText() string {
 	return helpText(&SyncToolsCommand{}, "juju sync-tools")
 }
 
-func (s *MainSuite) TestTearDown(c *gc.C) {
-	loggo.ResetLoggers()
-}
-
 func (s *MainSuite) TestRunMain(c *gc.C) {
 	defer testing.MakeSampleHome(c).Restore()
 	// The test array structure needs to be inline here as some of the
 	// expected values below use deployHelpText().  This constructs the deploy
 	// command and runs gets the help for it.  When the deploy command is
 	// setting the flags (which is needed for the help text) it is accessing
-	// config.JujuHome(), which panics if SetJujuHome has not been called.
+	// osenv.JujuHome(), which panics if SetJujuHome has not been called.
 	// The FakeHome from testing does this.
 	for i, t := range []struct {
 		summary string
@@ -181,7 +174,7 @@ var brokenConfig = `
 // breakJuju writes a dummy environment with incomplete configuration.
 // environMethod is called.
 func breakJuju(c *gc.C, environMethod string) (msg string) {
-	path := config.JujuHomePath("environments.yaml")
+	path := osenv.JujuHomePath("environments.yaml")
 	err := ioutil.WriteFile(path, []byte(brokenConfig), 0666)
 	c.Assert(err, gc.IsNil)
 	return `cannot parse "[^"]*": YAML error.*`
@@ -220,6 +213,7 @@ var commandNames = []string{
 	"add-relation",
 	"add-unit",
 	"api-endpoints",
+	"authorised-keys",
 	"bootstrap",
 	"debug-hooks",
 	"debug-log",
@@ -240,9 +234,13 @@ var commandNames = []string{
 	"help-tool",
 	"init",
 	"publish",
+	"remove-machine",  // alias for destroy-machine
 	"remove-relation", // alias for destroy-relation
+	"remove-service",  // alias for destroy-service
 	"remove-unit",     // alias for destroy-unit
 	"resolved",
+	"retry-provisioning",
+	"run",
 	"scp",
 	"set",
 	"set-constraints",
@@ -256,6 +254,8 @@ var commandNames = []string{
 	"terminate-machine", // alias for destroy-machine
 	"unexpose",
 	"unset",
+	"unset-env", // alias for unset-environment
+	"unset-environment",
 	"upgrade-charm",
 	"upgrade-juju",
 	"version",
@@ -264,7 +264,7 @@ var commandNames = []string{
 func (s *MainSuite) TestHelpCommands(c *gc.C) {
 	// Check that we have correctly registered all the commands
 	// by checking the help output.
-	defer config.SetJujuHome(config.SetJujuHome(c.MkDir()))
+	defer osenv.SetJujuHome(osenv.SetJujuHome(c.MkDir()))
 	out := badrun(c, 0, "help", "commands")
 	lines := strings.Split(out, "\n")
 	var names []string
@@ -280,16 +280,17 @@ func (s *MainSuite) TestHelpCommands(c *gc.C) {
 }
 
 var topicNames = []string{
-	"azure",
+	"azure-provider",
 	"basics",
 	"commands",
 	"constraints",
-	"ec2",
+	"ec2-provider",
 	"global-options",
 	"glossary",
-	"hpcloud",
-	"local",
-	"openstack",
+	"hpcloud-provider",
+	"local-provider",
+	"logging",
+	"openstack-provider",
 	"plugins",
 	"topics",
 }
@@ -297,7 +298,7 @@ var topicNames = []string{
 func (s *MainSuite) TestHelpTopics(c *gc.C) {
 	// Check that we have correctly registered all the topics
 	// by checking the help output.
-	defer config.SetJujuHome(config.SetJujuHome(c.MkDir()))
+	defer osenv.SetJujuHome(osenv.SetJujuHome(c.MkDir()))
 	out := badrun(c, 0, "help", "topics")
 	lines := strings.Split(out, "\n")
 	var names []string
@@ -316,8 +317,9 @@ var globalFlags = []string{
 	"--debug .*",
 	"--description .*",
 	"-h, --help .*",
-	"--log-config .*",
 	"--log-file .*",
+	"--logging-config .*",
+	"-q, --quiet .*",
 	"--show-log .*",
 	"-v, --verbose .*",
 }
@@ -325,7 +327,7 @@ var globalFlags = []string{
 func (s *MainSuite) TestHelpGlobalOptions(c *gc.C) {
 	// Check that we have correctly registered all the topics
 	// by checking the help output.
-	defer config.SetJujuHome(config.SetJujuHome(c.MkDir()))
+	defer osenv.SetJujuHome(osenv.SetJujuHome(c.MkDir()))
 	out := badrun(c, 0, "help", "global-options")
 	c.Assert(out, gc.Matches, `Global Options
 
