@@ -10,9 +10,9 @@ import (
 
 	"launchpad.net/gnuflag"
 
-	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
+	"launchpad.net/juju-core/state/api/params"
 )
 
 // SetCommand updates the configuration of a service.
@@ -59,8 +59,12 @@ func (c *SetCommand) Init(args []string) error {
 	return nil
 }
 
-// Run updates the configuration of a service.
-func (c *SetCommand) Run(ctx *cmd.Context) error {
+// serviceSet1dot16 does the final ServiceSet step using direct DB access
+// compatibility with an API server running 1.16 or older (when ServiceUnset
+// was not available). This fallback can be removed when we no longer maintain
+// 1.16 compatibility.
+// This was copied directly from the code in SetCommand.Run in 1.16
+func (c *SetCommand) serviceSet1dot16() error {
 	conn, err := juju.NewConnFromName(c.EnvName)
 	if err != nil {
 		return err
@@ -74,27 +78,41 @@ func (c *SetCommand) Run(ctx *cmd.Context) error {
 	if err != nil {
 		return err
 	}
-	var settings charm.Settings
-	if c.SettingsYAML.Path != "" {
-		settingsYAML, err := c.SettingsYAML.Read(ctx)
-		if err != nil {
-			return err
-		}
-		settings, err = ch.Config().ParseSettingsYAML(settingsYAML, c.ServiceName)
-		if err != nil {
-			return err
-		}
-	} else if len(c.SettingsStrings) > 0 {
-		settings, err = ch.Config().ParseSettingsStrings(c.SettingsStrings)
-		if err != nil {
-			return err
-		}
-	} else {
-		return nil
+	// We don't need the multiple logic here, because that should have
+	// already been taken care of by the API code (which *was* in 1.16).
+	settings, err := ch.Config().ParseSettingsStrings(c.SettingsStrings)
+	if err != nil {
+		return err
 	}
 	return service.UpdateConfigSettings(settings)
 }
 
+// Run updates the configuration of a service.
+func (c *SetCommand) Run(ctx *cmd.Context) error {
+	api, err := juju.NewAPIClientFromName(c.EnvName)
+	if err != nil {
+		return err
+	}
+	defer api.Close()
+
+	if c.SettingsYAML.Path != "" {
+		b, err := c.SettingsYAML.Read(ctx)
+		if err != nil {
+			return err
+		}
+		return api.ServiceSetYAML(c.ServiceName, string(b))
+	} else if len(c.SettingsStrings) == 0 {
+		return nil
+	}
+	err = api.ServiceSet(c.ServiceName, c.SettingsStrings)
+	if params.IsCodeNotImplemented(err) {
+		logger.Infof("NewServiceSetForClientAPI not supported by the API server, " +
+			"falling back to 1.16 compatibility mode (direct DB access)")
+		err = c.serviceSet1dot16()
+	}
+	return err
+}
+
 // parse parses the option k=v strings into a map of options to be
 // updated in the config. Keys with empty values are returned separately
 // and should be removed.
