@@ -7,6 +7,8 @@ import (
 	"errors"
 	"time"
 
+	"github.com/juju/loggo"
+	jc "github.com/juju/testing/checkers"
 	gc "launchpad.net/gocheck"
 
 	"launchpad.net/juju-core/cmd"
@@ -15,7 +17,6 @@ import (
 	"launchpad.net/juju-core/environs/sync"
 	"launchpad.net/juju-core/provider/dummy"
 	coretesting "launchpad.net/juju-core/testing"
-	jc "launchpad.net/juju-core/testing/checkers"
 	"launchpad.net/juju-core/testing/testbase"
 )
 
@@ -104,6 +105,13 @@ var syncToolsCommandTests = []struct {
 			DryRun: true,
 		},
 	},
+	{
+		description: "specific public",
+		args:        []string{"-e", "test-target", "--public"},
+		sctx: &sync.SyncContext{
+			Public: true,
+		},
+	},
 	{
 		description: "specify version",
 		args:        []string{"-e", "test-target", "--version", "1.2"},
@@ -117,7 +125,7 @@ var syncToolsCommandTests = []struct {
 func (s *syncToolsSuite) TestSyncToolsCommand(c *gc.C) {
 	for i, test := range syncToolsCommandTests {
 		c.Logf("test %d: %s", i, test.description)
-		targetEnv, err := environs.PrepareFromName("test-target", s.configStore)
+		targetEnv, err := environs.PrepareFromName("test-target", nullContext(c), s.configStore)
 		c.Assert(err, gc.IsNil)
 		called := false
 		syncTools = func(sctx *sync.SyncContext) error {
@@ -126,6 +134,7 @@ func (s *syncToolsSuite) TestSyncToolsCommand(c *gc.C) {
 			c.Assert(sctx.MinorVersion, gc.Equals, test.sctx.MinorVersion)
 			c.Assert(sctx.DryRun, gc.Equals, test.sctx.DryRun)
 			c.Assert(sctx.Dev, gc.Equals, test.sctx.Dev)
+			c.Assert(sctx.Public, gc.Equals, test.sctx.Public)
 			c.Assert(sctx.Source, gc.Equals, test.sctx.Source)
 			c.Assert(dummy.IsSameStorage(sctx.Target, targetEnv.Storage()), jc.IsTrue)
 			called = true
@@ -153,9 +162,41 @@ func (s *syncToolsSuite) TestSyncToolsCommandTargetDirectory(c *gc.C) {
 		called = true
 		return nil
 	}
-	ctx, err := runSyncToolsCommand(c, "-e", "test-target", "--destination", dir)
+	ctx, err := runSyncToolsCommand(c, "-e", "test-target", "--local-dir", dir)
 	c.Assert(err, gc.IsNil)
 	c.Assert(ctx, gc.NotNil)
 	c.Assert(called, jc.IsTrue)
 	s.Reset(c)
 }
+
+func (s *syncToolsSuite) TestSyncToolsCommandDeprecatedDestination(c *gc.C) {
+	called := false
+	dir := c.MkDir()
+	syncTools = func(sctx *sync.SyncContext) error {
+		c.Assert(sctx.AllVersions, gc.Equals, false)
+		c.Assert(sctx.DryRun, gc.Equals, false)
+		c.Assert(sctx.Dev, gc.Equals, false)
+		c.Assert(sctx.Source, gc.Equals, "")
+		url, err := sctx.Target.URL("")
+		c.Assert(err, gc.IsNil)
+		c.Assert(url, gc.Equals, "file://"+dir)
+		called = true
+		return nil
+	}
+	// Register writer.
+	tw := &loggo.TestWriter{}
+	c.Assert(loggo.RegisterWriter("deprecated-tester", tw, loggo.DEBUG), gc.IsNil)
+	defer loggo.RemoveWriter("deprecated-tester")
+	// Add deprecated message to be checked.
+	messages := []jc.SimpleMessage{
+		{loggo.WARNING, "Use of the --destination flag is deprecated in 1.18. Please use --local-dir instead."},
+	}
+	// Run sync-tools command with --destination flag.
+	ctx, err := runSyncToolsCommand(c, "-e", "test-target", "--destination", dir)
+	c.Assert(err, gc.IsNil)
+	c.Assert(ctx, gc.NotNil)
+	c.Assert(called, jc.IsTrue)
+	// Check deprecated message was logged.
+	c.Check(tw.Log, jc.LogMatches, messages)
+	s.Reset(c)
+}
