@@ -12,8 +12,10 @@ import (
 
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/cmd"
+	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/juju"
 	"launchpad.net/juju-core/names"
+	"launchpad.net/juju-core/state/api/params"
 )
 
 // UpgradeCharm is responsible for upgrading a service's charm.
@@ -32,13 +34,13 @@ revision available in the repository from which it was originally deployed. An
 explicit revision can be chosen with the --revision flag.
 
 If the charm came from a local repository, its path will be assumed to be
-$JUJU_REPOSITORY unless overridden by --repository. If there is no newer
-revision of a local charm directory, the local directory's revision will be
-automatically incremented to create a newer charm.
+$JUJU_REPOSITORY unless overridden by --repository.
 
 The local repository behaviour is tuned specifically to the workflow of a charm
 author working on a single client machine; use of local repositories from
-multiple clients is not supported and may lead to confusing behaviour.
+multiple clients is not supported and may lead to confusing behaviour. Each
+local charm gets uploaded with the revision specified in the charm, if possible,
+otherwise it gets a unique revision (highest in state + 1).
 
 The --switch flag allows you to replace the charm with an entirely different
 one. The new charm's URL and revision are inferred as they would be when running
@@ -49,7 +51,7 @@ information with which to determine compatibility; the operation will succeed,
 regardless of potential havoc, so long as the following conditions hold:
 
 - The new charm must declare all relations that the service is currently
-participating in. 
+participating in.
 - All config settings shared by the old and new charms must
 have the same types.
 
@@ -61,7 +63,7 @@ would specify revision number 5 of the wordpress charm.
 
 Use of the --force flag is not generally recommended; units upgraded while in an
 error state will not have upgrade-charm hooks executed, and may cause unexpected
-behavior. 
+behavior.
 `
 
 func (c *UpgradeCharmCommand) Info() *cmd.Info {
@@ -102,6 +104,80 @@ func (c *UpgradeCharmCommand) Init(args []string) error {
 // Run connects to the specified environment and starts the charm
 // upgrade process.
 func (c *UpgradeCharmCommand) Run(ctx *cmd.Context) error {
+	client, err := juju.NewAPIClientFromName(c.EnvName)
+	if err != nil {
+		return err
+	}
+	defer client.Close()
+	oldURL, err := client.ServiceGetCharmURL(c.ServiceName)
+	if params.IsCodeNotImplemented(err) {
+		logger.Infof("ServiceGetCharmURL is not implemented by the API server, switching to 1.16 compatibility mode (direct DB connection).")
+		return c.run1dot16(ctx)
+	}
+	if err != nil {
+		return err
+	}
+
+	attrs, err := client.EnvironmentGet()
+	if err != nil {
+		return err
+	}
+	conf, err := config.New(config.NoDefaults, attrs)
+	if err != nil {
+		return err
+	}
+
+	var newURL *charm.URL
+	if c.SwitchURL != "" {
+		newURL, err = resolveCharmURL(c.SwitchURL, client, conf)
+		if err != nil {
+			return err
+		}
+	} else {
+		// No new URL specified, but revision might have been.
+		newURL = oldURL.WithRevision(c.Revision)
+	}
+
+	repo, err := charm.InferRepository(newURL.Reference, ctx.AbsPath(c.RepoPath))
+	if err != nil {
+		return err
+	}
+	repo = config.SpecializeCharmRepo(repo, conf)
+
+	// If no explicit revision was set with either SwitchURL
+	// or Revision flags, discover the latest.
+	explicitRevision := true
+	if newURL.Revision == -1 {
+		explicitRevision = false
+		latest, err := charm.Latest(repo, newURL)
+		if err != nil {
+			return err
+		}
+		newURL = newURL.WithRevision(latest)
+	}
+	if *newURL == *oldURL {
+		if explicitRevision {
+			return fmt.Errorf("already running specified charm %q", newURL)
+		} else if newURL.Schema == "cs" {
+			// No point in trying to upgrade a charm store charm when
+			// we just determined that's the latest revision
+			// available.
+			return fmt.Errorf("already running latest charm %q", newURL)
+		}
+	}
+
+	addedURL, err := addCharmViaAPI(client, ctx, newURL, repo)
+	if err != nil {
+		return err
+	}
+
+	return client.ServiceSetCharm(c.ServiceName, addedURL.String(), c.Force)
+}
+
+// run1dot16 perfoms the charm upgrade using a 1.16 compatible code
+// path, with a direct state connection. Remove once the support for
+// 1.16 is dropped.
+func (c *UpgradeCharmCommand) run1dot16(ctx *cmd.Context) error {
 	conn, err := juju.NewConnFromName(c.EnvName)
 	if err != nil {
 		return err
@@ -111,15 +187,17 @@ func (c *UpgradeCharmCommand) Run(ctx *cmd.Context) error {
 	if err != nil {
 		return err
 	}
+
+	conf, err := conn.State.EnvironConfig()
+	if err != nil {
+		return err
+	}
+
 	oldURL, _ := service.CharmURL()
 	var newURL *charm.URL
 	if c.SwitchURL != "" {
 		// A new charm URL was explicitly specified.
-		conf, err := conn.State.EnvironConfig()
-		if err != nil {
-			return err
-		}
-		newURL, err = charm.InferURL(c.SwitchURL, conf.DefaultSeries())
+		newURL, err = resolveCharmURL1dot16(c.SwitchURL, conf)
 		if err != nil {
 			return err
 		}
@@ -127,16 +205,19 @@ func (c *UpgradeCharmCommand) Run(ctx *cmd.Context) error {
 		// No new URL specified, but revision might have been.
 		newURL = oldURL.WithRevision(c.Revision)
 	}
-	repo, err := charm.InferRepository(newURL, ctx.AbsPath(c.RepoPath))
+
+	repo, err := charm.InferRepository(newURL.Reference, ctx.AbsPath(c.RepoPath))
 	if err != nil {
 		return err
 	}
+	repo = config.SpecializeCharmRepo(repo, conf)
+
 	// If no explicit revision was set with either SwitchURL
 	// or Revision flags, discover the latest.
 	explicitRevision := true
 	if newURL.Revision == -1 {
 		explicitRevision = false
-		latest, err := repo.Latest(newURL)
+		latest, err := charm.Latest(repo, newURL)
 		if err != nil {
 			return err
 		}
