@@ -11,32 +11,32 @@ import (
 	"io/ioutil"
 	"strings"
 
+	jc "github.com/juju/testing/checkers"
 	gc "launchpad.net/gocheck"
 
+	"launchpad.net/juju-core/environs/config"
+	"launchpad.net/juju-core/environs/filestorage"
 	"launchpad.net/juju-core/environs/storage"
 	"launchpad.net/juju-core/environs/sync"
 	envtesting "launchpad.net/juju-core/environs/testing"
 	envtools "launchpad.net/juju-core/environs/tools"
 	"launchpad.net/juju-core/juju/testing"
 	coretesting "launchpad.net/juju-core/testing"
-	jc "launchpad.net/juju-core/testing/checkers"
-	coretools "launchpad.net/juju-core/tools"
 	"launchpad.net/juju-core/version"
 )
 
 type UpgradeJujuSuite struct {
 	testing.JujuConnSuite
+	toolsDir string
 }
 
 var _ = gc.Suite(&UpgradeJujuSuite{})
 
 var upgradeJujuTests = []struct {
 	about          string
-	private        []string
-	public         []string
+	tools          []string
 	currentVersion string
 	agentVersion   string
-	development    bool
 
 	args           []string
 	expectInitErr  string
@@ -48,11 +48,21 @@ var upgradeJujuTests = []struct {
 	currentVersion: "1.0.0-quantal-amd64",
 	args:           []string{"foo"},
 	expectInitErr:  "unrecognized args:.*",
+}, {
+	about:          "removed arg --dev specified",
+	currentVersion: "1.0.0-quantal-amd64",
+	args:           []string{"--dev"},
+	expectInitErr:  "flag provided but not defined: --dev",
 }, {
 	about:          "invalid --version value",
 	currentVersion: "1.0.0-quantal-amd64",
 	args:           []string{"--version", "invalid-version"},
 	expectInitErr:  "invalid version .*",
+}, {
+	about:          "just major version, no minor specified",
+	currentVersion: "4.2.0-quantal-amd64",
+	args:           []string{"--version", "4"},
+	expectInitErr:  `invalid version "4"`,
 }, {
 	about:          "major version upgrade to incompatible version",
 	currentVersion: "2.0.0-quantal-amd64",
@@ -81,52 +91,70 @@ var upgradeJujuTests = []struct {
 }, {
 	about:          "--upload-tools with inappropriate version 2",
 	currentVersion: "3.2.7-quantal-amd64",
-	args:           []string{"--upload-tools", "--version", "3.1.0.4"},
+	args:           []string{"--upload-tools", "--version", "3.2.8.4"},
 	expectInitErr:  "cannot specify build number when uploading tools",
 }, {
-	about:          "latest release from private storage",
-	private:        []string{"2.0.0-quantal-amd64", "2.0.2-quantal-i386", "2.0.3-quantal-amd64"},
-	public:         []string{"2.0.0-quantal-amd64", "2.0.4-quantal-amd64", "2.0.5-quantal-amd64"},
+	about:          "latest supported stable release",
+	tools:          []string{"2.2.0-quantal-amd64", "2.2.2-quantal-i386", "2.2.3-quantal-amd64"},
 	currentVersion: "2.0.0-quantal-amd64",
 	agentVersion:   "2.0.0",
-	expectVersion:  "2.0.3",
+	expectVersion:  "2.2.3",
 }, {
-	about:          "latest dev from private storage (because client is dev)",
-	private:        []string{"2.0.0-quantal-amd64", "2.2.0-quantal-amd64", "2.3.0-quantal-amd64", "3.0.1-quantal-amd64"},
-	public:         []string{"2.0.0-quantal-amd64", "2.4.0-quantal-amd64", "2.5.0-quantal-amd64"},
+	about:          "latest current release",
+	tools:          []string{"2.0.5-quantal-amd64", "2.0.1-quantal-i386", "2.3.3-quantal-amd64"},
+	currentVersion: "2.0.0-quantal-amd64",
+	agentVersion:   "2.0.0",
+	expectVersion:  "2.0.5",
+}, {
+	about:          "latest current release matching CLI, major version",
+	tools:          []string{"3.2.0-quantal-amd64"},
+	currentVersion: "3.2.0-quantal-amd64",
+	agentVersion:   "2.8.2",
+	expectVersion:  "3.2.0",
+}, {
+	about:          "latest current release matching CLI, major version, no matching major tools",
+	tools:          []string{"2.8.2-quantal-amd64"},
+	currentVersion: "3.2.0-quantal-amd64",
+	agentVersion:   "2.8.2",
+	expectErr:      "no matching tools available",
+}, {
+	about:          "latest current release matching CLI, major version, no matching tools",
+	tools:          []string{"3.3.0-quantal-amd64"},
+	currentVersion: "3.2.0-quantal-amd64",
+	agentVersion:   "2.8.2",
+	expectErr:      "no compatible tools available",
+}, {
+	about:          "no next supported available",
+	tools:          []string{"2.1.0-quantal-amd64", "2.1.5-quantal-i386", "2.3.3-quantal-amd64"},
+	currentVersion: "2.0.0-quantal-amd64",
+	agentVersion:   "2.0.0",
+	expectErr:      "no more recent supported versions available",
+}, {
+	about:          "latest supported stable, when client is dev",
+	tools:          []string{"2.1.1-quantal-amd64", "2.2.0-quantal-amd64", "2.3.0-quantal-amd64", "3.0.1-quantal-amd64"},
 	currentVersion: "2.1.0-quantal-amd64",
 	agentVersion:   "2.0.0",
-	expectVersion:  "2.3.0",
+	expectVersion:  "2.2.0",
 }, {
-	about:          "latest dev from private storage (because agent is dev)",
-	private:        []string{"2.0.0-quantal-amd64", "2.2.0-quantal-amd64", "2.3.0-quantal-amd64", "3.0.1-quantal-amd64"},
-	public:         []string{"2.0.0-quantal-amd64", "2.4.0-quantal-amd64", "2.5.0-quantal-amd64"},
+	about:          "latest current, when agent is dev",
+	tools:          []string{"2.1.1-quantal-amd64", "2.2.0-quantal-amd64", "2.3.0-quantal-amd64", "3.0.1-quantal-amd64"},
 	currentVersion: "2.0.0-quantal-amd64",
 	agentVersion:   "2.1.0",
-	expectVersion:  "2.3.0",
-}, {
-	about:          "latest dev from private storage (because --dev flag)",
-	private:        []string{"2.0.0-quantal-amd64", "2.2.0-quantal-amd64", "2.3.0-quantal-amd64"},
-	public:         []string{"2.0.0-quantal-amd64", "2.4.0-quantal-amd64", "2.5.0-quantal-amd64"},
-	currentVersion: "2.0.0-quantal-amd64",
-	args:           []string{"--dev"},
-	agentVersion:   "2.0.0",
-	expectVersion:  "2.3.0",
-}, {
-	about:          "latest dev from private storage (because dev env setting)",
-	private:        []string{"2.0.0-quantal-amd64", "2.2.0-quantal-amd64", "2.3.0-quantal-amd64"},
-	public:         []string{"2.0.0-quantal-amd64", "2.4.0-quantal-amd64", "2.5.0-quantal-amd64"},
-	currentVersion: "2.0.0-quantal-amd64",
-	development:    true,
-	agentVersion:   "2.0.0",
-	expectVersion:  "2.3.0",
+	expectVersion:  "2.2.0",
 }, {
 	about:          "specified version",
-	private:        []string{"2.3.0-quantal-amd64"},
+	tools:          []string{"2.3.0-quantal-amd64"},
 	currentVersion: "2.0.0-quantal-amd64",
 	agentVersion:   "2.0.0",
 	args:           []string{"--version", "2.3.0"},
 	expectVersion:  "2.3.0",
+}, {
+	about:          "specified major version",
+	tools:          []string{"3.2.0-quantal-amd64"},
+	currentVersion: "3.2.0-quantal-amd64",
+	agentVersion:   "2.8.2",
+	args:           []string{"--version", "3.2.0"},
+	expectVersion:  "3.2.0",
 }, {
 	about:          "specified version missing, but already set",
 	currentVersion: "3.0.0-quantal-amd64",
@@ -138,65 +166,58 @@ var upgradeJujuTests = []struct {
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--version", "3.2.0"},
-	expectErr:      "no matching tools available",
+	expectErr:      "no tools available",
 }, {
 	about:          "specified version, no matching major version",
-	private:        []string{"4.2.0-quantal-amd64"},
+	tools:          []string{"4.2.0-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--version", "3.2.0"},
 	expectErr:      "no matching tools available",
 }, {
 	about:          "specified version, no matching minor version",
-	private:        []string{"3.4.0-quantal-amd64"},
+	tools:          []string{"3.4.0-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--version", "3.2.0"},
 	expectErr:      "no matching tools available",
 }, {
 	about:          "specified version, no matching patch version",
-	private:        []string{"3.2.5-quantal-amd64"},
+	tools:          []string{"3.2.5-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--version", "3.2.0"},
 	expectErr:      "no matching tools available",
 }, {
 	about:          "specified version, no matching build version",
-	private:        []string{"3.2.0.2-quantal-amd64"},
+	tools:          []string{"3.2.0.2-quantal-amd64"},
 	currentVersion: "3.0.0-quantal-amd64",
 	agentVersion:   "3.0.0",
 	args:           []string{"--version", "3.2.0"},
 	expectErr:      "no matching tools available",
 }, {
 	about:          "major version downgrade to incompatible version",
-	private:        []string{"3.2.0-quantal-amd64"},
+	tools:          []string{"3.2.0-quantal-amd64"},
 	currentVersion: "3.2.0-quantal-amd64",
 	agentVersion:   "4.2.0",
 	args:           []string{"--version", "3.2.0"},
-	expectErr:      "cannot change major version from 4 to 3",
+	expectErr:      "cannot change version from 4.2.0 to 3.2.0",
 }, {
-	about:          "major version upgrade to compatible version",
-	private:        []string{"3.2.0-quantal-amd64"},
+	about:          "minor version downgrade to incompatible version",
+	tools:          []string{"3.2.0-quantal-amd64"},
 	currentVersion: "3.2.0-quantal-amd64",
-	agentVersion:   "2.8.2",
+	agentVersion:   "3.3.0",
 	args:           []string{"--version", "3.2.0"},
-	expectErr:      "major version upgrades are not supported yet",
+	expectErr:      "cannot change version from 3.3.0 to 3.2.0",
 }, {
-	about:          "nothing available 1",
+	about:          "nothing available",
 	currentVersion: "2.0.0-quantal-amd64",
 	agentVersion:   "2.0.0",
 	expectVersion:  "2.0.0",
 }, {
 	about:          "nothing available 2",
 	currentVersion: "2.0.0-quantal-amd64",
-	public:         []string{"3.2.0-quantal-amd64"},
-	agentVersion:   "2.0.0",
-	expectVersion:  "2.0.0",
-}, {
-	about:          "nothing available 3",
-	currentVersion: "2.0.0-quantal-amd64",
-	private:        []string{"3.2.0-quantal-amd64"},
-	public:         []string{"3.4.0-quantal-amd64"},
+	tools:          []string{"3.2.0-quantal-amd64"},
 	agentVersion:   "2.0.0",
 	expectVersion:  "2.0.0",
 }, {
@@ -205,14 +226,14 @@ var upgradeJujuTests = []struct {
 	agentVersion:   "2.0.0",
 	args:           []string{"--upload-tools"},
 	expectVersion:  "2.2.0.1",
-	expectUploaded: []string{"2.2.0.1-quantal-amd64", "2.2.0.1-precise-amd64", "2.2.0.1-raring-amd64"},
+	expectUploaded: []string{"2.2.0.1-quantal-amd64", "2.2.0.1-%LTS%-amd64", "2.2.0.1-raring-amd64"},
 }, {
 	about:          "upload with explicit version",
 	currentVersion: "2.2.0-quantal-amd64",
 	agentVersion:   "2.0.0",
 	args:           []string{"--upload-tools", "--version", "2.7.3"},
 	expectVersion:  "2.7.3.1",
-	expectUploaded: []string{"2.7.3.1-quantal-amd64", "2.7.3.1-precise-amd64", "2.7.3.1-raring-amd64"},
+	expectUploaded: []string{"2.7.3.1-quantal-amd64", "2.7.3.1-%LTS%-amd64", "2.7.3.1-raring-amd64"},
 }, {
 	about:          "upload with explicit series",
 	currentVersion: "2.2.0-quantal-amd64",
@@ -226,65 +247,63 @@ var upgradeJujuTests = []struct {
 	agentVersion:   "2.0.0",
 	args:           []string{"--upload-tools"},
 	expectVersion:  "2.1.0.1",
-	expectUploaded: []string{"2.1.0.1-quantal-amd64", "2.1.0.1-precise-amd64", "2.1.0.1-raring-amd64"},
+	expectUploaded: []string{"2.1.0.1-quantal-amd64", "2.1.0.1-%LTS%-amd64", "2.1.0.1-raring-amd64"},
 }, {
 	about:          "upload bumps version when necessary",
-	private:        []string{"2.4.6-quantal-amd64", "2.4.8-quantal-amd64"},
-	public:         []string{"2.4.6.4-quantal-amd64"}, //ignored
+	tools:          []string{"2.4.6-quantal-amd64", "2.4.8-quantal-amd64"},
 	currentVersion: "2.4.6-quantal-amd64",
 	agentVersion:   "2.4.0",
 	args:           []string{"--upload-tools"},
 	expectVersion:  "2.4.6.1",
-	expectUploaded: []string{"2.4.6.1-quantal-amd64", "2.4.6.1-precise-amd64", "2.4.6.1-raring-amd64"},
+	expectUploaded: []string{"2.4.6.1-quantal-amd64", "2.4.6.1-%LTS%-amd64", "2.4.6.1-raring-amd64"},
 }, {
 	about:          "upload re-bumps version when necessary",
-	private:        []string{"2.4.6-quantal-amd64", "2.4.6.2-saucy-i386", "2.4.8-quantal-amd64"},
-	public:         []string{"2.4.6.10-quantal-amd64"}, //ignored
+	tools:          []string{"2.4.6-quantal-amd64", "2.4.6.2-saucy-i386", "2.4.8-quantal-amd64"},
 	currentVersion: "2.4.6-quantal-amd64",
 	agentVersion:   "2.4.6.2",
 	args:           []string{"--upload-tools"},
 	expectVersion:  "2.4.6.3",
-	expectUploaded: []string{"2.4.6.3-quantal-amd64", "2.4.6.3-precise-amd64", "2.4.6.3-raring-amd64"},
+	expectUploaded: []string{"2.4.6.3-quantal-amd64", "2.4.6.3-%LTS%-amd64", "2.4.6.3-raring-amd64"},
 }, {
 	about:          "upload with explicit version bumps when necessary",
 	currentVersion: "2.2.0-quantal-amd64",
-	private:        []string{"2.7.3.1-quantal-amd64"},
+	tools:          []string{"2.7.3.1-quantal-amd64"},
 	agentVersion:   "2.0.0",
 	args:           []string{"--upload-tools", "--version", "2.7.3"},
 	expectVersion:  "2.7.3.2",
-	expectUploaded: []string{"2.7.3.2-quantal-amd64", "2.7.3.2-precise-amd64", "2.7.3.2-raring-amd64"},
+	expectUploaded: []string{"2.7.3.2-quantal-amd64", "2.7.3.2-%LTS%-amd64", "2.7.3.2-raring-amd64"},
 }}
 
-// mockUploadTools simulates the effect of tools.Upload, but skips the time-
-// consuming build from source.
-// TODO(fwereade) better factor agent/tools such that build logic is
-// exposed and can itself be neatly mocked?
-func mockUploadTools(stor storage.Storage, forceVersion *version.Number, series ...string) (*coretools.Tools, error) {
-	vers := version.Current
-	if forceVersion != nil {
-		vers.Number = *forceVersion
-	}
-	versions := []version.Binary{vers}
-	for _, series := range series {
-		if series != version.Current.Series {
-			newVers := vers
-			newVers.Series = series
-			versions = append(versions, newVers)
+// getMockBuildTools returns a sync.BuildToolsTarballFunc implementation which generates
+// a fake tools tarball.
+func (s *UpgradeJujuSuite) getMockBuildTools(c *gc.C) sync.BuildToolsTarballFunc {
+	return func(forceVersion *version.Number) (*sync.BuiltTools, error) {
+		// UploadFakeToolsVersions requires a storage to write to.
+		stor, err := filestorage.NewFileStorageWriter(s.toolsDir)
+		c.Assert(err, gc.IsNil)
+		vers := version.Current
+		if forceVersion != nil {
+			vers.Number = *forceVersion
 		}
+		versions := []version.Binary{vers}
+		uploadedTools, err := envtesting.UploadFakeToolsVersions(stor, versions...)
+		c.Assert(err, gc.IsNil)
+		agentTools := uploadedTools[0]
+		return &sync.BuiltTools{
+			Dir:         s.toolsDir,
+			StorageName: envtools.StorageName(vers),
+			Version:     vers,
+			Size:        agentTools.Size,
+			Sha256Hash:  agentTools.SHA256,
+		}, nil
 	}
-	agentTools, err := envtesting.UploadFakeToolsVersions(stor, versions...)
-	if err != nil {
-		return nil, err
-	}
-	return agentTools[0], nil
 }
 
 func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
+	s.PatchValue(&sync.BuildToolsTarball, s.getMockBuildTools(c))
 	oldVersion := version.Current
-	uploadTools = mockUploadTools
 	defer func() {
 		version.Current = oldVersion
-		uploadTools = sync.Upload
 	}()
 
 	for i, test := range upgradeJujuTests {
@@ -304,27 +323,24 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 		}
 
 		// Set up state and environ, and run the command.
-		cfg, err := s.State.EnvironConfig()
+		toolsDir := c.MkDir()
+		updateAttrs := map[string]interface{}{
+			"agent-version":      test.agentVersion,
+			"tools-metadata-url": "file://" + toolsDir,
+		}
+		err := s.State.UpdateEnvironConfig(updateAttrs, nil, nil)
 		c.Assert(err, gc.IsNil)
-		cfg, err = cfg.Apply(map[string]interface{}{
-			"agent-version": test.agentVersion,
-			"development":   test.development,
-		})
-		c.Assert(err, gc.IsNil)
-		err = s.State.SetEnvironConfig(cfg)
-		c.Assert(err, gc.IsNil)
-		versions := make([]version.Binary, len(test.private))
-		for i, v := range test.private {
+		versions := make([]version.Binary, len(test.tools))
+		for i, v := range test.tools {
 			versions[i] = version.MustParseBinary(v)
+		}
+		if len(versions) > 0 {
+			envtesting.MustUploadFakeToolsVersions(s.Conn.Environ.Storage(), versions...)
+			stor, err := filestorage.NewFileStorageWriter(toolsDir)
+			c.Assert(err, gc.IsNil)
+			envtesting.MustUploadFakeToolsVersions(stor, versions...)
+		}
 
-		}
-		envtesting.MustUploadFakeToolsVersions(s.Conn.Environ.Storage(), versions...)
-		versions = make([]version.Binary, len(test.public))
-		for i, v := range test.public {
-			versions[i] = version.MustParseBinary(v)
-		}
-		stor := s.Conn.Environ.PublicStorage().(storage.Storage)
-		envtesting.MustUploadFakeToolsVersions(stor, versions...)
 		err = com.Run(coretesting.Context(c))
 		if test.expectErr != "" {
 			c.Check(err, gc.ErrorMatches, test.expectErr)
@@ -334,14 +350,16 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 		}
 
 		// Check expected changes to environ/state.
-		cfg, err = s.State.EnvironConfig()
+		cfg, err := s.State.EnvironConfig()
 		c.Check(err, gc.IsNil)
 		agentVersion, ok := cfg.AgentVersion()
 		c.Check(ok, gc.Equals, true)
 		c.Check(agentVersion, gc.Equals, version.MustParse(test.expectVersion))
-		c.Check(cfg.Development(), gc.Equals, test.development)
 
 		for _, uploaded := range test.expectUploaded {
+			// Substitute latest LTS for placeholder in expected series for uploaded tools
+			uploaded = strings.Replace(uploaded, "%LTS%", config.LatestLtsSeries(), 1)
+
 			vers := version.MustParseBinary(uploaded)
 			r, err := storage.Get(s.Conn.Environ.Storage(), envtools.StorageName(vers))
 			if !c.Check(err, gc.IsNil) {
@@ -350,7 +368,9 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 			data, err := ioutil.ReadAll(r)
 			r.Close()
 			c.Check(err, gc.IsNil)
-			checkToolsContent(c, data, "jujud contents "+uploaded)
+			expectContent := version.Current
+			expectContent.Number = agentVersion
+			checkToolsContent(c, data, "jujud contents "+expectContent.String())
 		}
 	}
 }
@@ -388,16 +408,13 @@ func checkToolsContent(c *gc.C, data []byte, uploaded string) {
 func (s *UpgradeJujuSuite) Reset(c *gc.C) {
 	s.JujuConnSuite.Reset(c)
 	envtesting.RemoveTools(c, s.Conn.Environ.Storage())
-	envtesting.RemoveTools(c, s.Conn.Environ.PublicStorage().(storage.Storage))
-	cfg, err := s.State.EnvironConfig()
-	c.Assert(err, gc.IsNil)
-	cfg, err = cfg.Apply(map[string]interface{}{
+	updateAttrs := map[string]interface{}{
 		"default-series": "raring",
 		"agent-version":  "1.2.3",
-	})
-	c.Assert(err, gc.IsNil)
-	err = s.State.SetEnvironConfig(cfg)
+	}
+	err := s.State.UpdateEnvironConfig(updateAttrs, nil, nil)
 	c.Assert(err, gc.IsNil)
+	s.toolsDir = c.MkDir()
 }
 
 func (s *UpgradeJujuSuite) TestUpgradeJujuWithRealUpload(c *gc.C) {
