@@ -12,6 +12,7 @@ import (
 
 	"launchpad.net/juju-core/agent"
 	"launchpad.net/juju-core/cmd"
+	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/log"
 	"launchpad.net/juju-core/state"
@@ -19,9 +20,11 @@ import (
 	apiagent "launchpad.net/juju-core/state/api/agent"
 	apideployer "launchpad.net/juju-core/state/api/deployer"
 	"launchpad.net/juju-core/state/api/params"
+	apirsyslog "launchpad.net/juju-core/state/api/rsyslog"
 	"launchpad.net/juju-core/version"
 	"launchpad.net/juju-core/worker"
 	"launchpad.net/juju-core/worker/deployer"
+	"launchpad.net/juju-core/worker/rsyslog"
 	"launchpad.net/juju-core/worker/upgrader"
 )
 
@@ -38,6 +41,10 @@ type AgentConf struct {
 
 // addFlags injects common agent flags into f.
 func (c *AgentConf) addFlags(f *gnuflag.FlagSet) {
+	// TODO(dimitern) 2014-02-19 bug 1282025
+	// We need to pass a config location here instead and
+	// use it to locate the conf and the infer the data-dir
+	// from there instead of passing it like that.
 	f.StringVar(&c.dataDir, "data-dir", "/var/lib/juju", "directory for juju data")
 }
 
@@ -49,7 +56,7 @@ func (c *AgentConf) checkArgs(args []string) error {
 }
 
 func (c *AgentConf) read(tag string) (err error) {
-	c.config, err = agent.ReadConf(c.dataDir, tag)
+	c.config, err = agent.ReadConf(agent.ConfigPath(c.dataDir, tag))
 	return
 }
 
@@ -103,9 +110,10 @@ func (e *fatalError) Error() string {
 }
 
 func isFatal(err error) bool {
-	isTerminate := err == worker.ErrTerminateAgent
-	notProvisioned := params.IsCodeNotProvisioned(err)
-	if isTerminate || notProvisioned || isUpgraded(err) {
+	if err == worker.ErrTerminateAgent {
+		return true
+	}
+	if isUpgraded(err) {
 		return true
 	}
 	_, ok := err.(*fatalError)
@@ -126,7 +134,7 @@ func connectionIsFatal(conn pinger) func(err error) bool {
 			return true
 		}
 		if err := conn.Ping(); err != nil {
-			log.Infof("error pinging %T: %v", conn, err)
+			logger.Infof("error pinging %T: %v", conn, err)
 			return true
 		}
 		return false
@@ -146,7 +154,7 @@ func isleep(d time.Duration, stop <-chan struct{}) bool {
 }
 
 func openState(agentConfig agent.Config, a Agent) (*state.State, AgentState, error) {
-	st, err := agentConfig.OpenState()
+	st, err := agentConfig.OpenState(environs.NewStatePolicy())
 	if err != nil {
 		return nil, nil, err
 	}
@@ -161,7 +169,11 @@ func openState(agentConfig agent.Config, a Agent) (*state.State, AgentState, err
 	return st, entity, nil
 }
 
-func openAPIState(agentConfig agent.Config, a Agent) (*api.State, *apiagent.Entity, error) {
+type apiOpener interface {
+	OpenAPI(api.DialOpts) (*api.State, string, error)
+}
+
+func openAPIState(agentConfig apiOpener, a Agent) (*api.State, *apiagent.Entity, error) {
 	// We let the API dial fail immediately because the
 	// runner's loop outside the caller of openAPIState will
 	// keep on retrying. If we block for ages here,
@@ -169,6 +181,9 @@ func openAPIState(agentConfig agent.Config, a Agent) (*api.State, *apiagent.Enti
 	// be interrupted.
 	st, newPassword, err := agentConfig.OpenAPI(api.DialOpts{})
 	if err != nil {
+		if params.IsCodeNotProvisioned(err) {
+			err = worker.ErrTerminateAgent
+		}
 		if params.IsCodeUnauthorized(err) {
 			err = worker.ErrTerminateAgent
 		}
@@ -201,7 +216,7 @@ func agentDone(err error) error {
 	if ug, ok := err.(*upgrader.UpgradeReadyError); ok {
 		if err := ug.ChangeAgentTools(); err != nil {
 			// Return and let upstart deal with the restart.
-			return err
+			return log.LoggedErrorf(logger, "cannot change agent tools: %v", err)
 		}
 	}
 	return err
@@ -228,7 +243,7 @@ func (c *closeWorker) Kill() {
 func (c *closeWorker) Wait() error {
 	err := c.worker.Wait()
 	if err := c.closer.Close(); err != nil {
-		log.Errorf("closeWorker: close error: %v", err)
+		logger.Errorf("closeWorker: close error: %v", err)
 	}
 	return err
 }
@@ -241,3 +256,19 @@ func (c *closeWorker) Wait() error {
 var newDeployContext = func(st *apideployer.State, agentConfig agent.Config) deployer.Context {
 	return deployer.NewSimpleContext(agentConfig, st)
 }
+
+// newRsyslogConfigWorker creates and returns a new RsyslogConfigWorker
+// based on the specified configuration parameters.
+var newRsyslogConfigWorker = func(st *apirsyslog.State, agentConfig agent.Config, mode rsyslog.RsyslogMode) (worker.Worker, error) {
+	tag := agentConfig.Tag()
+	namespace := agentConfig.Value(agent.Namespace)
+	var addrs []string
+	if mode == rsyslog.RsyslogModeForwarding {
+		var err error
+		addrs, err = agentConfig.APIAddresses()
+		if err != nil {
+			return nil, err
+		}
+	}
+	return rsyslog.NewRsyslogConfigWorker(st, mode, tag, namespace, addrs)
+}
