@@ -6,8 +6,6 @@ package main
 import (
 	"encoding/base64"
 	"fmt"
-	"io/ioutil"
-	"strings"
 
 	"launchpad.net/gnuflag"
 	"launchpad.net/goyaml"
@@ -15,23 +13,20 @@ import (
 	"launchpad.net/juju-core/agent"
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/constraints"
-	"launchpad.net/juju-core/environs/bootstrap"
-	"launchpad.net/juju-core/environs/cloudinit"
+	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/environs/config"
 	"launchpad.net/juju-core/instance"
-	"launchpad.net/juju-core/provider/common"
 	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/state/api/params"
 )
 
-// Cloud-init write the URL to be used to load the bootstrap state into this file.
-// A variable is used here to allow tests to override.
-var providerStateURLFile = cloudinit.BootstrapStateURLFile
-
 type BootstrapCommand struct {
 	cmd.CommandBase
 	Conf        AgentConf
 	EnvConfig   map[string]interface{}
 	Constraints constraints.Value
+	Hardware    instance.HardwareCharacteristics
+	InstanceId  string
 }
 
 // Info returns a decription of the command.
@@ -46,6 +41,8 @@ func (c *BootstrapCommand) SetFlags(f *gnuflag.FlagSet) {
 	c.Conf.addFlags(f)
 	yamlBase64Var(f, &c.EnvConfig, "env-config", "", "initial environment configuration (yaml, base64 encoded)")
 	f.Var(constraints.ConstraintsValue{&c.Constraints}, "constraints", "initial environment constraints (space-separated strings)")
+	f.Var(&c.Hardware, "hardware", "hardware characteristics (space-separated strings)")
+	f.StringVar(&c.InstanceId, "instance-id", "", "unique instance-id for bootstrap machine")
 }
 
 // Init initializes the command for running.
@@ -53,60 +50,42 @@ func (c *BootstrapCommand) Init(args []string) error {
 	if len(c.EnvConfig) == 0 {
 		return requiredError("env-config")
 	}
+	if c.InstanceId == "" {
+		return requiredError("instance-id")
+	}
 	return c.Conf.checkArgs(args)
 }
 
 // Run initializes state for an environment.
 func (c *BootstrapCommand) Run(_ *cmd.Context) error {
-	err := c.Conf.read("bootstrap")
+	envCfg, err := config.New(config.NoDefaults, c.EnvConfig)
 	if err != nil {
 		return err
 	}
-	cfg, err := config.New(config.NoDefaults, c.EnvConfig)
+	if err := c.Conf.read("machine-0"); err != nil {
+		return err
+	}
+	// agent.Jobs is an optional field in the agent config, and was
+	// introduced after 1.17.2. We default to allowing units on
+	// machine-0 if missing.
+	jobs := c.Conf.config.Jobs()
+	if len(jobs) == 0 {
+		jobs = []params.MachineJob{
+			params.JobManageEnviron,
+			params.JobHostUnits,
+		}
+	}
+	st, _, err := c.Conf.config.InitializeState(envCfg, agent.BootstrapMachineConfig{
+		Constraints:     c.Constraints,
+		Jobs:            jobs,
+		InstanceId:      instance.Id(c.InstanceId),
+		Characteristics: c.Hardware,
+	}, state.DefaultDialOpts(), environs.NewStatePolicy())
 	if err != nil {
 		return err
 	}
-
-	// There is no entity that's created at init time
-	st, err := agent.InitialStateConfiguration(c.Conf.config, cfg, state.DefaultDialOpts())
-	if err != nil {
-		return err
-	}
-	defer st.Close()
-
-	// TODO(fwereade): we need to be able to customize machine jobs,
-	// not just hardcode these values; in particular, JobHostUnits
-	// on a machine, like this one, that is running JobManageEnviron
-	// (not to mention the actual state server itself...) will allow
-	// a malicious or compromised unit to trivially access to the
-	// user's environment credentials. However, given that this point
-	// is currently moot (see Upgrader in this package), the pseudo-
-	// local provider mode (in which everything is deployed with
-	// `--to 0`) offers enough value to enough people that
-	// JobHostUnits is currently always enabled. This will one day
-	// have to change, but it's strictly less important than fixing
-	// Upgrader, and it's a capability we'll always want to have
-	// available for the aforementioned use case.
-	jobs := []state.MachineJob{
-		state.JobManageEnviron, state.JobManageState, state.JobHostUnits,
-	}
-
-	data, err := ioutil.ReadFile(providerStateURLFile)
-	if err != nil {
-		return fmt.Errorf("cannot read provider-state-url file: %v", err)
-	}
-	stateInfoURL := strings.Split(string(data), "\n")[0]
-	bsState, err := common.LoadStateFromURL(stateInfoURL)
-	if err != nil {
-		return fmt.Errorf("cannot load state from URL %q (read from %q): %v", stateInfoURL, providerStateURLFile, err)
-	}
-	instId := bsState.StateInstances[0]
-	var characteristics instance.HardwareCharacteristics
-	if len(bsState.Characteristics) > 0 {
-		characteristics = bsState.Characteristics[0]
-	}
-
-	return bootstrap.ConfigureBootstrapMachine(st, c.Constraints, c.Conf.dataDir, jobs, instance.Id(instId), characteristics)
+	st.Close()
+	return nil
 }
 
 // yamlBase64Value implements gnuflag.Value on a map[string]interface{}.
