@@ -5,24 +5,23 @@ package main
 
 import (
 	"encoding/base64"
-	"io/ioutil"
-	"path/filepath"
 
+	jc "github.com/juju/testing/checkers"
 	gc "launchpad.net/gocheck"
 	"launchpad.net/goyaml"
 
 	"launchpad.net/juju-core/agent"
 	"launchpad.net/juju-core/constraints"
-	"launchpad.net/juju-core/environs/jujutest"
+	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/errors"
 	"launchpad.net/juju-core/instance"
-	"launchpad.net/juju-core/provider/common"
 	"launchpad.net/juju-core/provider/dummy"
 	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/testing"
-	jc "launchpad.net/juju-core/testing/checkers"
 	"launchpad.net/juju-core/testing/testbase"
 	"launchpad.net/juju-core/utils"
+	"launchpad.net/juju-core/version"
 )
 
 // We don't want to use JujuConnSuite because it gives us
@@ -30,31 +29,15 @@ import (
 type BootstrapSuite struct {
 	testbase.LoggingSuite
 	testing.MgoSuite
-	dataDir              string
-	providerStateURLFile string
+	dataDir string
+	logDir  string
 }
 
 var _ = gc.Suite(&BootstrapSuite{})
 
-var testRoundTripper = &jujutest.ProxyRoundTripper{}
-
-func init() {
-	// Prepare mock http transport for provider-state output in tests.
-	testRoundTripper.RegisterForScheme("test")
-}
-
 func (s *BootstrapSuite) SetUpSuite(c *gc.C) {
 	s.LoggingSuite.SetUpSuite(c)
 	s.MgoSuite.SetUpSuite(c)
-	stateInfo := common.BootstrapState{
-		StateInstances: []instance.Id{instance.Id("dummy.instance.id")},
-	}
-	stateData, err := goyaml.Marshal(stateInfo)
-	c.Assert(err, gc.IsNil)
-	content := map[string]string{"/" + common.StateFile: string(stateData)}
-	testRoundTripper.Sub = jujutest.NewCannedRoundTripper(content, nil)
-	s.providerStateURLFile = filepath.Join(c.MkDir(), "provider-state-url")
-	providerStateURLFile = s.providerStateURLFile
 }
 
 func (s *BootstrapSuite) TearDownSuite(c *gc.C) {
@@ -66,6 +49,7 @@ func (s *BootstrapSuite) SetUpTest(c *gc.C) {
 	s.LoggingSuite.SetUpTest(c)
 	s.MgoSuite.SetUpTest(c)
 	s.dataDir = c.MkDir()
+	s.logDir = c.MkDir()
 }
 
 func (s *BootstrapSuite) TearDownTest(c *gc.C) {
@@ -76,21 +60,29 @@ func (s *BootstrapSuite) TearDownTest(c *gc.C) {
 var testPassword = "my-admin-secret"
 
 func testPasswordHash() string {
-	return utils.PasswordHash(testPassword)
+	return utils.UserPasswordHash(testPassword, utils.CompatSalt)
 }
 
-func (s *BootstrapSuite) initBootstrapCommand(c *gc.C, args ...string) (machineConf agent.Config, cmd *BootstrapCommand, err error) {
-	ioutil.WriteFile(s.providerStateURLFile, []byte("test://localhost/provider-state\n"), 0600)
+func (s *BootstrapSuite) initBootstrapCommand(c *gc.C, jobs []params.MachineJob, args ...string) (machineConf agent.Config, cmd *BootstrapCommand, err error) {
+	if len(jobs) == 0 {
+		// Add default jobs.
+		jobs = []params.MachineJob{
+			params.JobManageEnviron, params.JobHostUnits,
+		}
+	}
 	// NOTE: the old test used an equivalent of the NewAgentConfig, but it
 	// really should be using NewStateMachineConfig.
 	params := agent.AgentConfigParams{
-		DataDir:        s.dataDir,
-		Tag:            "bootstrap",
-		Password:       testPasswordHash(),
-		Nonce:          state.BootstrapNonce,
-		StateAddresses: []string{testing.MgoAddr},
-		APIAddresses:   []string{"0.1.2.3:1234"},
-		CACert:         []byte(testing.CACert),
+		LogDir:            s.logDir,
+		DataDir:           s.dataDir,
+		Jobs:              jobs,
+		Tag:               "bootstrap",
+		UpgradedToVersion: version.Current.Number,
+		Password:          testPasswordHash(),
+		Nonce:             state.BootstrapNonce,
+		StateAddresses:    []string{testing.MgoServer.Addr()},
+		APIAddresses:      []string{"0.1.2.3:1234"},
+		CACert:            []byte(testing.CACert),
 	}
 	bootConf, err := agent.NewAgentConfig(params)
 	c.Assert(err, gc.IsNil)
@@ -109,16 +101,17 @@ func (s *BootstrapSuite) initBootstrapCommand(c *gc.C, args ...string) (machineC
 }
 
 func (s *BootstrapSuite) TestInitializeEnvironment(c *gc.C) {
-	_, cmd, err := s.initBootstrapCommand(c, "--env-config", testConfig)
+	hw := instance.MustParseHardware("arch=amd64 mem=8G")
+	_, cmd, err := s.initBootstrapCommand(c, nil, "--env-config", testConfig, "--instance-id", "anything", "--hardware", hw.String())
 	c.Assert(err, gc.IsNil)
 	err = cmd.Run(nil)
 	c.Assert(err, gc.IsNil)
 
 	st, err := state.Open(&state.Info{
-		Addrs:    []string{testing.MgoAddr},
+		Addrs:    []string{testing.MgoServer.Addr()},
 		CACert:   []byte(testing.CACert),
 		Password: testPasswordHash(),
-	}, state.DefaultDialOpts())
+	}, state.DefaultDialOpts(), environs.NewStatePolicy())
 	c.Assert(err, gc.IsNil)
 	defer st.Close()
 	machines, err := st.AllMachines()
@@ -127,7 +120,12 @@ func (s *BootstrapSuite) TestInitializeEnvironment(c *gc.C) {
 
 	instid, err := machines[0].InstanceId()
 	c.Assert(err, gc.IsNil)
-	c.Assert(instid, gc.Equals, instance.Id("dummy.instance.id"))
+	c.Assert(instid, gc.Equals, instance.Id("anything"))
+
+	stateHw, err := machines[0].HardwareCharacteristics()
+	c.Assert(err, gc.IsNil)
+	c.Assert(stateHw, gc.NotNil)
+	c.Assert(*stateHw, gc.DeepEquals, hw)
 
 	cons, err := st.EnvironConstraints()
 	c.Assert(err, gc.IsNil)
@@ -136,16 +134,20 @@ func (s *BootstrapSuite) TestInitializeEnvironment(c *gc.C) {
 
 func (s *BootstrapSuite) TestSetConstraints(c *gc.C) {
 	tcons := constraints.Value{Mem: uint64p(2048), CpuCores: uint64p(2)}
-	_, cmd, err := s.initBootstrapCommand(c, "--env-config", testConfig, "--constraints", tcons.String())
+	_, cmd, err := s.initBootstrapCommand(c, nil,
+		"--env-config", testConfig,
+		"--instance-id", "anything",
+		"--constraints", tcons.String(),
+	)
 	c.Assert(err, gc.IsNil)
 	err = cmd.Run(nil)
 	c.Assert(err, gc.IsNil)
 
 	st, err := state.Open(&state.Info{
-		Addrs:    []string{testing.MgoAddr},
+		Addrs:    []string{testing.MgoServer.Addr()},
 		CACert:   []byte(testing.CACert),
 		Password: testPasswordHash(),
-	}, state.DefaultDialOpts())
+	}, state.DefaultDialOpts(), environs.NewStatePolicy())
 	c.Assert(err, gc.IsNil)
 	defer st.Close()
 	cons, err := st.EnvironConstraints()
@@ -164,28 +166,48 @@ func uint64p(v uint64) *uint64 {
 	return &v
 }
 
-func (s *BootstrapSuite) TestMachinerWorkers(c *gc.C) {
-	_, cmd, err := s.initBootstrapCommand(c, "--env-config", testConfig)
+func (s *BootstrapSuite) TestDefaultMachineJobs(c *gc.C) {
+	expectedJobs := []state.MachineJob{
+		state.JobManageEnviron, state.JobHostUnits,
+	}
+	_, cmd, err := s.initBootstrapCommand(c, nil, "--env-config", testConfig, "--instance-id", "anything")
 	c.Assert(err, gc.IsNil)
 	err = cmd.Run(nil)
 	c.Assert(err, gc.IsNil)
 
 	st, err := state.Open(&state.Info{
-		Addrs:    []string{testing.MgoAddr},
+		Addrs:    []string{testing.MgoServer.Addr()},
 		CACert:   []byte(testing.CACert),
 		Password: testPasswordHash(),
-	}, state.DefaultDialOpts())
+	}, state.DefaultDialOpts(), environs.NewStatePolicy())
 	c.Assert(err, gc.IsNil)
 	defer st.Close()
 	m, err := st.Machine("0")
 	c.Assert(err, gc.IsNil)
-	c.Assert(m.Jobs(), gc.DeepEquals, []state.MachineJob{
-		state.JobManageEnviron, state.JobManageState, state.JobHostUnits,
-	})
+	c.Assert(m.Jobs(), gc.DeepEquals, expectedJobs)
+}
+
+func (s *BootstrapSuite) TestConfiguredMachineJobs(c *gc.C) {
+	jobs := []params.MachineJob{params.JobManageEnviron}
+	_, cmd, err := s.initBootstrapCommand(c, jobs, "--env-config", testConfig, "--instance-id", "anything")
+	c.Assert(err, gc.IsNil)
+	err = cmd.Run(nil)
+	c.Assert(err, gc.IsNil)
+
+	st, err := state.Open(&state.Info{
+		Addrs:    []string{testing.MgoServer.Addr()},
+		CACert:   []byte(testing.CACert),
+		Password: testPasswordHash(),
+	}, state.DefaultDialOpts(), environs.NewStatePolicy())
+	c.Assert(err, gc.IsNil)
+	defer st.Close()
+	m, err := st.Machine("0")
+	c.Assert(err, gc.IsNil)
+	c.Assert(m.Jobs(), gc.DeepEquals, []state.MachineJob{state.JobManageEnviron})
 }
 
 func testOpenState(c *gc.C, info *state.Info, expectErrType error) {
-	st, err := state.Open(info, state.DefaultDialOpts())
+	st, err := state.Open(info, state.DefaultDialOpts(), environs.NewStatePolicy())
 	if st != nil {
 		st.Close()
 	}
@@ -197,7 +219,7 @@ func testOpenState(c *gc.C, info *state.Info, expectErrType error) {
 }
 
 func (s *BootstrapSuite) TestInitialPassword(c *gc.C) {
-	machineConf, cmd, err := s.initBootstrapCommand(c, "--env-config", testConfig)
+	machineConf, cmd, err := s.initBootstrapCommand(c, nil, "--env-config", testConfig, "--instance-id", "anything")
 	c.Assert(err, gc.IsNil)
 
 	err = cmd.Run(nil)
@@ -206,14 +228,14 @@ func (s *BootstrapSuite) TestInitialPassword(c *gc.C) {
 	// Check that we cannot now connect to the state without a
 	// password.
 	info := &state.Info{
-		Addrs:  []string{testing.MgoAddr},
+		Addrs:  []string{testing.MgoServer.Addr()},
 		CACert: []byte(testing.CACert),
 	}
 	testOpenState(c, info, errors.Unauthorizedf(""))
 
 	// Check we can log in to mongo as admin.
 	info.Tag, info.Password = "", testPasswordHash()
-	st, err := state.Open(info, state.DefaultDialOpts())
+	st, err := state.Open(info, state.DefaultDialOpts(), environs.NewStatePolicy())
 	c.Assert(err, gc.IsNil)
 	// Reset password so the tests can continue to use the same server.
 	defer st.Close()
@@ -228,51 +250,83 @@ func (s *BootstrapSuite) TestInitialPassword(c *gc.C) {
 	// Check that the machine configuration has been given a new
 	// password and that we can connect to mongo as that machine
 	// and that the in-mongo password also verifies correctly.
-	machineConf1, err := agent.ReadConf(machineConf.DataDir(), "machine-0")
+	machineConf1, err := agent.ReadConf(agent.ConfigPath(machineConf.DataDir(), "machine-0"))
 	c.Assert(err, gc.IsNil)
 
-	st, err = machineConf1.OpenState()
+	st, err = machineConf1.OpenState(environs.NewStatePolicy())
 	c.Assert(err, gc.IsNil)
 	defer st.Close()
 }
 
-var base64ConfigTests = []struct {
-	input    []string
-	err      string
-	expected map[string]interface{}
+var bootstrapArgTests = []struct {
+	input              []string
+	err                string
+	expectedInstanceId string
+	expectedHardware   instance.HardwareCharacteristics
+	expectedConfig     map[string]interface{}
 }{
 	{
-		// no value supplied
-		nil,
-		"--env-config option must be set",
-		nil,
+		// no value supplied for env-config
+		err: "--env-config option must be set",
 	}, {
-		// empty
-		[]string{"--env-config", ""},
-		"--env-config option must be set",
-		nil,
+		// empty env-config
+		input: []string{"--env-config", ""},
+		err:   "--env-config option must be set",
 	}, {
 		// wrong, should be base64
-		[]string{"--env-config", "name: banana\n"},
-		".*illegal base64 data at input byte.*",
-		nil,
+		input: []string{"--env-config", "name: banana\n"},
+		err:   ".*illegal base64 data at input byte.*",
 	}, {
-		[]string{"--env-config", base64.StdEncoding.EncodeToString([]byte("name: banana\n"))},
-		"",
-		map[string]interface{}{"name": "banana"},
+		// no value supplied for instance-id
+		input: []string{
+			"--env-config", base64.StdEncoding.EncodeToString([]byte("name: banana\n")),
+		},
+		err: "--instance-id option must be set",
+	}, {
+		// empty instance-id
+		input: []string{
+			"--env-config", base64.StdEncoding.EncodeToString([]byte("name: banana\n")),
+			"--instance-id", "",
+		},
+		err: "--instance-id option must be set",
+	}, {
+		input: []string{
+			"--env-config", base64.StdEncoding.EncodeToString([]byte("name: banana\n")),
+			"--instance-id", "anything",
+		},
+		expectedInstanceId: "anything",
+		expectedConfig:     map[string]interface{}{"name": "banana"},
+	}, {
+		input: []string{
+			"--env-config", base64.StdEncoding.EncodeToString([]byte("name: banana\n")),
+			"--instance-id", "anything",
+			"--hardware", "nonsense",
+		},
+		err: `invalid value "nonsense" for flag --hardware: malformed characteristic "nonsense"`,
+	}, {
+		input: []string{
+			"--env-config", base64.StdEncoding.EncodeToString([]byte("name: banana\n")),
+			"--instance-id", "anything",
+			"--hardware", "arch=amd64 cpu-cores=4 root-disk=2T",
+		},
+		expectedInstanceId: "anything",
+		expectedHardware:   instance.MustParseHardware("arch=amd64 cpu-cores=4 root-disk=2T"),
+		expectedConfig:     map[string]interface{}{"name": "banana"},
 	},
 }
 
-func (s *BootstrapSuite) TestBase64Config(c *gc.C) {
-	for i, t := range base64ConfigTests {
+func (s *BootstrapSuite) TestBootstrapArgs(c *gc.C) {
+	for i, t := range bootstrapArgTests {
 		c.Logf("test %d", i)
 		var args []string
 		args = append(args, t.input...)
-		_, cmd, err := s.initBootstrapCommand(c, args...)
+		_, cmd, err := s.initBootstrapCommand(c, nil, args...)
 		if t.err == "" {
 			c.Assert(cmd, gc.NotNil)
 			c.Assert(err, gc.IsNil)
-			c.Assert(cmd.EnvConfig, gc.DeepEquals, t.expected)
+			c.Assert(cmd.EnvConfig, gc.DeepEquals, t.expectedConfig)
+			c.Assert(cmd.InstanceId, gc.Equals, t.expectedInstanceId)
+			c.Assert(cmd.Hardware, gc.DeepEquals, t.expectedHardware)
 		} else {
 			c.Assert(err, gc.ErrorMatches, t.err)
 		}
