@@ -5,16 +5,19 @@ package main
 
 import (
 	"fmt"
+	"runtime"
 
+	"github.com/juju/loggo"
 	"launchpad.net/gnuflag"
-	"launchpad.net/loggo"
 	"launchpad.net/tomb"
 
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/version"
 	"launchpad.net/juju-core/worker"
-	"launchpad.net/juju-core/worker/logger"
+	workerlogger "launchpad.net/juju-core/worker/logger"
+	"launchpad.net/juju-core/worker/rsyslog"
 	"launchpad.net/juju-core/worker/uniter"
 	"launchpad.net/juju-core/worker/upgrader"
 )
@@ -27,7 +30,7 @@ type UnitAgent struct {
 	tomb     tomb.Tomb
 	Conf     AgentConf
 	UnitName string
-	runner   *worker.Runner
+	runner   worker.Runner
 }
 
 // Info returns usage information for the command.
@@ -70,7 +73,7 @@ func (a *UnitAgent) Run(ctx *cmd.Context) error {
 	if err := a.Conf.read(a.Tag()); err != nil {
 		return err
 	}
-	agentLogger.Infof("unit agent %v start", a.Tag())
+	agentLogger.Infof("unit agent %v start (%s [%s])", a.Tag(), version.Current, runtime.Compiler)
 	a.runner.StartWorker("api", a.APIWorkers)
 	err := agentDone(a.runner.Wait())
 	a.tomb.Kill(err)
@@ -89,11 +92,14 @@ func (a *UnitAgent) APIWorkers() (worker.Worker, error) {
 		return upgrader.NewUpgrader(st.Upgrader(), agentConfig), nil
 	})
 	runner.StartWorker("logger", func() (worker.Worker, error) {
-		return logger.NewLogger(st.Logger(), agentConfig), nil
+		return workerlogger.NewLogger(st.Logger(), agentConfig), nil
 	})
 	runner.StartWorker("uniter", func() (worker.Worker, error) {
 		return uniter.NewUniter(st.Uniter(), entity.Tag(), dataDir), nil
 	})
+	runner.StartWorker("rsyslog", func() (worker.Worker, error) {
+		return newRsyslogConfigWorker(st.Rsyslog(), agentConfig, rsyslog.RsyslogModeForwarding)
+	})
 	return newCloseWorker(runner, st), nil
 }
 
