@@ -10,16 +10,22 @@ import (
 
 	"launchpad.net/juju-core/agent"
 	"launchpad.net/juju-core/cmd"
+	envtesting "launchpad.net/juju-core/environs/testing"
+	jujutesting "launchpad.net/juju-core/juju/testing"
 	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
-	"launchpad.net/juju-core/testing"
+	apirsyslog "launchpad.net/juju-core/state/api/rsyslog"
+	coretesting "launchpad.net/juju-core/testing"
 	"launchpad.net/juju-core/tools"
+	"launchpad.net/juju-core/version"
 	"launchpad.net/juju-core/worker"
+	"launchpad.net/juju-core/worker/rsyslog"
+	"launchpad.net/juju-core/worker/upgrader"
 )
 
 type UnitSuite struct {
-	testing.GitSuite
+	coretesting.GitSuite
 	agentSuite
 }
 
@@ -35,19 +41,26 @@ func (s *UnitSuite) TearDownTest(c *gc.C) {
 	s.GitSuite.TearDownTest(c)
 }
 
-const initialUnitPassword = "unit-password"
+const initialUnitPassword = "unit-password-1234567890"
 
 // primeAgent creates a unit, and sets up the unit agent's directory.
-// It returns the new unit and the agent's configuration.
-func (s *UnitSuite) primeAgent(c *gc.C) (*state.Unit, agent.Config, *tools.Tools) {
-	svc, err := s.State.AddService("wordpress", s.AddTestingCharm(c, "wordpress"))
-	c.Assert(err, gc.IsNil)
+// It returns the assigned machine, new unit and the agent's configuration.
+func (s *UnitSuite) primeAgent(c *gc.C) (*state.Machine, *state.Unit, agent.Config, *tools.Tools) {
+	jujutesting.AddStateServerMachine(c, s.State)
+	svc := s.AddTestingService(c, "wordpress", s.AddTestingCharm(c, "wordpress"))
 	unit, err := svc.AddUnit()
 	c.Assert(err, gc.IsNil)
 	err = unit.SetPassword(initialUnitPassword)
 	c.Assert(err, gc.IsNil)
-	conf, tools := s.agentSuite.primeAgent(c, unit.Tag(), initialUnitPassword)
-	return unit, conf, tools
+	// Assign the unit to a machine.
+	err = unit.AssignToNewMachine()
+	c.Assert(err, gc.IsNil)
+	id, err := unit.AssignedMachineId()
+	c.Assert(err, gc.IsNil)
+	machine, err := s.State.Machine(id)
+	c.Assert(err, gc.IsNil)
+	conf, tools := s.agentSuite.primeAgent(c, unit.Tag(), initialUnitPassword, version.Current)
+	return machine, unit, conf, tools
 }
 
 func (s *UnitSuite) newAgent(c *gc.C, unit *state.Unit) *UnitAgent {
@@ -97,7 +110,7 @@ func waitForUnitStarted(stateConn *state.State, unit *state.Unit, c *gc.C) {
 		select {
 		case <-timeout:
 			c.Fatalf("no activity detected")
-		case <-time.After(testing.ShortWait):
+		case <-time.After(coretesting.ShortWait):
 			err := unit.Refresh()
 			c.Assert(err, gc.IsNil)
 			st, info, data, err := unit.Status()
@@ -120,7 +133,7 @@ func waitForUnitStarted(stateConn *state.State, unit *state.Unit, c *gc.C) {
 }
 
 func (s *UnitSuite) TestRunStop(c *gc.C) {
-	unit, _, _ := s.primeAgent(c)
+	_, unit, _, _ := s.primeAgent(c)
 	a := s.newAgent(c, unit)
 	go func() { c.Check(a.Run(nil), gc.IsNil) }()
 	defer func() { c.Check(a.Stop(), gc.IsNil) }()
@@ -128,13 +141,37 @@ func (s *UnitSuite) TestRunStop(c *gc.C) {
 }
 
 func (s *UnitSuite) TestUpgrade(c *gc.C) {
-	unit, _, currentTools := s.primeAgent(c)
-	a := s.newAgent(c, unit)
-	s.testUpgrade(c, a, unit.Tag(), currentTools)
+	machine, unit, _, currentTools := s.primeAgent(c)
+	agent := s.newAgent(c, unit)
+	newVers := version.Current
+	newVers.Patch++
+	envtesting.AssertUploadFakeToolsVersions(c, s.Conn.Environ.Storage(), newVers)
+
+	// Set the machine agent version to trigger an upgrade.
+	err := machine.SetAgentVersion(newVers)
+	c.Assert(err, gc.IsNil)
+	err = runWithTimeout(agent)
+	envtesting.CheckUpgraderReadyError(c, err, &upgrader.UpgradeReadyError{
+		AgentName: unit.Tag(),
+		OldTools:  currentTools.Version,
+		NewTools:  newVers,
+		DataDir:   s.DataDir(),
+	})
+}
+
+func (s *UnitSuite) TestUpgradeFailsWithoutTools(c *gc.C) {
+	machine, unit, _, _ := s.primeAgent(c)
+	agent := s.newAgent(c, unit)
+	newVers := version.Current
+	newVers.Patch++
+	err := machine.SetAgentVersion(newVers)
+	c.Assert(err, gc.IsNil)
+	err = runWithTimeout(agent)
+	c.Assert(err, gc.ErrorMatches, "timed out waiting for agent to finish.*")
 }
 
 func (s *UnitSuite) TestWithDeadUnit(c *gc.C) {
-	unit, _, _ := s.primeAgent(c)
+	_, unit, _, _ := s.primeAgent(c)
 	err := unit.EnsureDead()
 	c.Assert(err, gc.IsNil)
 	a := s.newAgent(c, unit)
@@ -150,12 +187,12 @@ func (s *UnitSuite) TestWithDeadUnit(c *gc.C) {
 }
 
 func (s *UnitSuite) TestOpenAPIState(c *gc.C) {
-	unit, _, _ := s.primeAgent(c)
+	_, unit, _, _ := s.primeAgent(c)
 	s.testOpenAPIState(c, unit, s.newAgent(c, unit), initialUnitPassword)
 }
 
 func (s *UnitSuite) TestOpenAPIStateWithBadCredsTerminates(c *gc.C) {
-	conf, _ := s.agentSuite.primeAgent(c, "unit-missing-0", "no-password")
+	conf, _ := s.agentSuite.primeAgent(c, "unit-missing-0", "no-password", version.Current)
 	_, _, err := openAPIState(conf, nil)
 	c.Assert(err, gc.Equals, worker.ErrTerminateAgent)
 }
@@ -173,7 +210,7 @@ func (f *fakeUnitAgent) Tag() string {
 }
 
 func (s *UnitSuite) TestOpenAPIStateWithDeadEntityTerminates(c *gc.C) {
-	unit, conf, _ := s.primeAgent(c)
+	_, unit, conf, _ := s.primeAgent(c)
 	err := unit.EnsureDead()
 	c.Assert(err, gc.IsNil)
 	_, _, err = openAPIState(conf, &fakeUnitAgent{"wordpress/0"})
@@ -183,7 +220,7 @@ func (s *UnitSuite) TestOpenAPIStateWithDeadEntityTerminates(c *gc.C) {
 func (s *UnitSuite) TestOpenStateFails(c *gc.C) {
 	// Start a unit agent and make sure it doesn't set a mongo password
 	// we can use to connect to state with.
-	unit, conf, _ := s.primeAgent(c)
+	_, unit, conf, _ := s.primeAgent(c)
 	a := s.newAgent(c, unit)
 	go func() { c.Check(a.Run(nil), gc.IsNil) }()
 	defer func() { c.Check(a.Stop(), gc.IsNil) }()
@@ -191,3 +228,23 @@ func (s *UnitSuite) TestOpenStateFails(c *gc.C) {
 
 	s.assertCannotOpenState(c, conf.Tag(), conf.DataDir())
 }
+
+func (s *UnitSuite) TestRsyslogConfigWorker(c *gc.C) {
+	created := make(chan rsyslog.RsyslogMode, 1)
+	s.PatchValue(&newRsyslogConfigWorker, func(_ *apirsyslog.State, _ agent.Config, mode rsyslog.RsyslogMode) (worker.Worker, error) {
+		created <- mode
+		return worker.NewRunner(isFatal, moreImportant), nil
+	})
+
+	_, unit, _, _ := s.primeAgent(c)
+	a := s.newAgent(c, unit)
+	go func() { c.Check(a.Run(nil), gc.IsNil) }()
+	defer func() { c.Check(a.Stop(), gc.IsNil) }()
+
+	select {
+	case <-time.After(coretesting.LongWait):
+		c.Fatalf("timeout while waiting for rsyslog worker to be created")
+	case mode := <-created:
+		c.Assert(mode, gc.Equals, rsyslog.RsyslogModeForwarding)
+	}
+}
